@@ -1,26 +1,42 @@
 
-
 /*
 #    Sfront, a SAOL to C translator    
 #    This file: Handles asys interfaces
-#    Copyright (C) 1999  Regents of the University of California
 #
-#    This program is free software; you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License (Version 2) as
-#    published by the Free Software Foundation.
+# Copyright (c) 1999-2006, Regents of the University of California
+# All rights reserved.
 #
-#    This program is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#    GNU General Public License for more details.
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are
+# met:
 #
-#    You should have received a copy of the GNU General Public License
-#    along with this program; if not, write to the Free Software
-#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#  Redistributions of source code must retain the above copyright
+#  notice, this list of conditions and the following disclaimer.
+#
+#  Redistributions in binary form must reproduce the above copyright
+#  notice, this list of conditions and the following disclaimer in the
+#  documentation and/or other materials provided with the distribution.
+#
+#  Neither the name of the University of California, Berkeley nor the
+#  names of its contributors may be used to endorse or promote products
+#  derived from this software without specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #
 #    Maintainer: John Lazzaro, lazzaro@cs.berkeley.edu
 */
 
+
 #include "tree.h"
 
 #define PORTAUDIO_OSS_TESTING      0  /* 1 to add Portaudio OSS driver */
@@ -58,23 +74,29 @@ void printaudiohelp(void)
 {
 
   printf("Where C output binary sends its audio:\n");
-  printf("       [-aout fname.wav]   WAV file fname.wav\n");
-  printf("       [-aout fname.aif]   AIFF file fname.aif\n");
-  printf("       [-aout fname.raw]   16-bit integers to fname.raw\n");
-  printf("       [-aout fname.dat]   View .dat files\n");
-  printf("       [-aout std]         stdout\n");
-  printf("       [-aout null]        do nothing with the output\n");
-  printf("       [-aout linux]       soundcard under Linux (OSS).\n");
-  printf("       [-aout pa_win_wmme] soundcard under Windows (WMME API).\n");
-  printf("       [-aout pa_win_ds]   soundcard under Windows (Direct API).\n");
-  printf("       [-aout coreaudio]   soundcard under Mac OS X (CoreAudio).\n");
-  printf("       [-aout freebsd]     soundcard under FreeBSD.\n");
-  printf("       [-aout irix]        soundcard under IRIX (6.2 or later).\n");
-  printf("       [-aout hpux]        soundcard under HPUX.\n");
+  printf("       [-aout fname.wav]    WAV file fname.wav (16-bit)\n");
+  printf("       [-aout fname.wav-24] WAV file fname.wav (24-bit)\n");
+  printf("       [-aout fname.wav-16] WAV file fname.wav (16-bit)\n");
+  printf("       [-aout fname.wav-8]  WAV file fname.wav (8-bit)\n");
+  printf("       [-aout fname.aif]    AIFF file fname.aif (16-bit)\n");
+  printf("       [-aout fname.aif-24] AIFF file fname.aif (24-bit)\n");
+  printf("       [-aout fname.aif-16] AIFF file fname.aif (16-bit)\n");
+  printf("       [-aout fname.aif-8]  AIFF file fname.aif (8-bit)\n");
+  printf("       [-aout fname.raw]    16-bit integers to fname.raw\n");
+  printf("       [-aout fname.dat]    View .dat files\n");
+  printf("       [-aout std]          stdout\n");
+  printf("       [-aout null]         do nothing with the output\n");
+  printf("       [-aout linux]        soundcard under Linux (OSS).\n");
+  printf("       [-aout pa_win_wmme]  soundcard under Windows (WMME API).\n");
+  printf("       [-aout pa_win_ds]    soundcard under Windows (Direct API).\n");
+  printf("       [-aout coreaudio]    soundcard under Mac OS X (CoreAudio).\n");
+  printf("       [-aout freebsd]      soundcard under FreeBSD.\n");
+  printf("       [-aout irix]         soundcard under IRIX (6.2 or later).\n");
+  printf("       [-aout hpux]         soundcard under HPUX.\n");
   if (PORTAUDIO_OSS_TESTING)
-    printf("       [-aout pa_unix_oss] soundcard under UNIX OSS systems.\n");
+    printf("       [-aout pa_unix_oss]  soundcard under UNIX OSS systems.\n");
   if (DEPRECIATED_DSOUND_TESTING)
-    printf("       [-aout dsound]      depreciated Directsound driver.\n");
+    printf("       [-aout dsound]       depreciated Directsound driver.\n");
   printf("Where C output binary gets its audio:\n");
   printf("       [-ain fname.wav]    WAV file fname.wav\n");
   printf("       [-ain fname.aif]    AIFF file fname.aif\n");
@@ -101,6 +123,7 @@ int aoutfilecheck(char * fname)
 
 {
   int i, found;
+  char * suffix;
 
   aoutname = dupval(fname);
 
@@ -244,16 +267,98 @@ int aoutfilecheck(char * fname)
       aoutflow = PASSIVE_FLOW;
       return 0;
     }
-  if (strstr(fname,".wav") != NULL)
+  if ((suffix = strstr(fname,".wav")) != NULL)
     {
       aout = DRIVER_WAV;
       aoutflow = PASSIVE_FLOW;
-      return 0;
+
+      if (strlen(suffix) == 4)  /* no wordsize specified */
+	return 0;
+
+      if ((strlen(suffix) == 5) || (strlen(suffix) >= 8) || (suffix[4] != '-')) 
+	{
+	  printf("Error: Audio output filename syntax -aout %s is incorrect.\n",
+		 fname); 
+	  printf("       Try filename.wav-16 or filename.wav-24 or filename.wav-8.\n\n");
+
+	  noerrorplace();  /* never returns */
+	}
+
+      *(strrchr(aoutname, '-')) = '\0';   /* can never return null */
+
+      if ((strlen(suffix) == 6) && (suffix[5] == '8'))
+	{
+	  outfile_wordsize = WORDSIZE_8BIT;
+	  return 0;
+	}
+
+      if ((strlen(suffix) == 7) && (suffix[5] == '1') && (suffix[6] == '6'))
+	{
+	  outfile_wordsize = WORDSIZE_16BIT;
+	  return 0;
+	}
+
+      if ((strlen(suffix) == 7) && (suffix[5] == '2') && (suffix[6] == '4'))
+	{
+	  outfile_wordsize = WORDSIZE_24BIT;
+	  return 0;
+	}
+
+      printf("Error: Audio output filename syntax -aout %s is incorrect.\n",
+	     fname);
+      printf("       Try filename.wav-16 or filename.wav-24 or filename.wav-8.\n\n");
+
+      noerrorplace();  /* never returns */
     }
-  if (strstr(fname,".aif") != NULL)
+  if ((suffix = strstr(fname,".aif")) != NULL)
     {
       aout = DRIVER_AIF;
       aoutflow = PASSIVE_FLOW;
+
+      if (strstr(fname,".aiff"))   /* solves parsing "aiff" below */
+	{
+	  suffix = strstr(fname,".aiff");
+	  suffix++;
+	}
+
+      if (strlen(suffix) == 4)  /* no wordsize specified */
+	return 0;
+
+      if ((strlen(suffix) == 5) || (strlen(suffix) >= 8) || (suffix[4] != '-')) 
+	{
+	  printf("Error: Audio output filename syntax -aout %s is incorrect.\n",
+		 fname);
+	  printf("       Try filename.aif-16 or filename.aif-24 or filename.aif-8.\n\n");
+
+	  noerrorplace();  /* never returns */
+	}
+
+      *(strrchr(aoutname, '-')) = '\0';   /* can never return null */
+
+      if ((strlen(suffix) == 6) && (suffix[5] == '8'))
+	{
+	  outfile_wordsize = WORDSIZE_8BIT;
+	  return 0;
+	}
+
+      if ((strlen(suffix) == 7) && (suffix[5] == '1') && (suffix[6] == '6'))
+	{
+	  outfile_wordsize = WORDSIZE_16BIT;
+	  return 0;
+	}
+
+      if ((strlen(suffix) == 7) && (suffix[5] == '2') && (suffix[6] == '4'))
+	{
+	  outfile_wordsize = WORDSIZE_24BIT;
+	  return 0;
+	}
+
+      printf("Error: -aout filename specification syntax %s is incorrect.\n",
+	     fname); 
+      printf("       Try filename.aif-16 or filename.aif-24 or filename.aif-8.\n\n");
+
+      noerrorplace();  /* never returns */
+
       return 0;
     }
   if (strstr(fname,".dat") != NULL)
@@ -431,6 +536,8 @@ int makeaudiotypeout(int anum)
   case DRIVER_PA_WIN_DS:
   case DRIVER_COREAUDIO:
   case DRIVER_VCDAT:
+  case DRIVER_WAV:
+  case DRIVER_AIF:
     ret = SAMPLE_FLOAT;
     break;
   default:
@@ -454,6 +561,8 @@ int makeaudiotypein(int anum)
   case DRIVER_PA_WIN_DS:
   case DRIVER_COREAUDIO:
   case DRIVER_VCDAT:
+  case DRIVER_WAV:
+  case DRIVER_AIF:
     ret = SAMPLE_FLOAT;
     break;
   default:
@@ -928,7 +1037,7 @@ int ainparams_wav_getlong(FILE * fd, int numbytes, unsigned long * ret)
 /*          reads the header of a wave file              */
 /*********************************************************/
 
-FILE * ainparams_wav_reader(FILE * fd, int * srate, int * inchannels)
+int ainparams_wav_reader(FILE * fd, int * srate, int * inchannels)
 
 {
   unsigned long i, cookie;
@@ -937,39 +1046,40 @@ FILE * ainparams_wav_reader(FILE * fd, int * srate, int * inchannels)
   if ((ainparams_wav_soundtypecheck(fd, "RIFF")!= AINPARAMS_WAV_MATCH) ||
       (ainparams_wav_flushbytes(fd, 4)!= AINPARAMS_WAV_DONE) ||
       (ainparams_wav_soundtypecheck(fd, "WAVE")!= AINPARAMS_WAV_MATCH))
-    return fd;
+    return AINPARAMS_WAV_ERROR;
   
   while ((cookie = ainparams_wav_soundtypecheck(fd, "fmt "))
 	 != AINPARAMS_WAV_MATCH)
     {
       if ((cookie == AINPARAMS_WAV_EOF) || 
 	  (ainparams_wav_getlong(fd, 4, &i) != AINPARAMS_WAV_DONE))
-	return fd;
+	return AINPARAMS_WAV_ERROR;
       if (ainparams_wav_flushbytes(fd, i)!= AINPARAMS_WAV_DONE)
-	return fd;
+	return AINPARAMS_WAV_ERROR;
     }
 
   if (ainparams_wav_getlong(fd, 4, &i) != AINPARAMS_WAV_DONE)
-    return fd;
+    return AINPARAMS_WAV_ERROR;
 
   len = i;
   if (((len -= 16) < 0) ||
       (ainparams_wav_getlong(fd, 2, &i) != AINPARAMS_WAV_DONE))
-    return fd;
+    return AINPARAMS_WAV_ERROR;
 
   if (i != 1)   /* PCM magic number */
-    return fd;
+    return AINPARAMS_WAV_ERROR;
 
   if (ainparams_wav_getlong(fd, 2, &i) != AINPARAMS_WAV_DONE)
-    return fd;
+    return AINPARAMS_WAV_ERROR;
 
   *inchannels = i;
 
   if (ainparams_wav_getlong(fd, 4, &i) != AINPARAMS_WAV_DONE)
-    return fd;
+    return AINPARAMS_WAV_ERROR;
   
   *srate = i;
-  return fd;
+
+  return AINPARAMS_WAV_DONE;
 }
 
 
@@ -988,10 +1098,17 @@ void ainparams_wav(int * srate, int * inchannels)
     fd = fopen("output.wav","rb");
   
   if (!fd)
-    return;
+    {
+      fprintf(stderr, "Warning: -ain WAV file %s cannot be opened.\n",
+	      ainname ? ainname : "(default name: output.wav)");
+      return;
+    }
 
-  fclose(ainparams_wav_reader(fd, srate, inchannels));
+  if (ainparams_wav_reader(fd, srate, inchannels) == AINPARAMS_WAV_ERROR)
+    fprintf(stderr, "Warning: -ain WAV file %s cannot be parsed.\n",
+	    ainname ? ainname : "(default name: output.wav)");
 
+  fclose(fd);
 }
 
 #undef AINPARAMS_WAV_MATCH  
@@ -1138,10 +1255,10 @@ int ainparams_aif_getslong(FILE * fd, int numbytes, long * ret)
 }
     
 /*********************************************************/
-/*          reads the header of a wave file              */
+/*          reads the header of an AIF file              */
 /*********************************************************/
 
-FILE * ainparams_aif_reader(FILE * fd, int * srate, int * inchannels)
+int ainparams_aif_reader(FILE * fd, int * srate, int * inchannels)
 
 {
   unsigned long i, m;
@@ -1150,53 +1267,50 @@ FILE * ainparams_aif_reader(FILE * fd, int * srate, int * inchannels)
 
 
   if (ainparams_aif_soundtypecheck(fd,"FORM")!= AINPARAMS_AIF_MATCH)
-    return fd;
+    return AINPARAMS_AIF_ERROR;
   if (ainparams_aif_flushbytes(fd, 4)!= AINPARAMS_AIF_DONE)
-    return fd;
+    return AINPARAMS_AIF_ERROR;
   if (ainparams_aif_soundtypecheck(fd, "AIFF")!= AINPARAMS_AIF_MATCH)
-    return fd;
+    return AINPARAMS_AIF_ERROR;
   if (ainparams_aif_getbytes(fd, &c[0],4)!= AINPARAMS_AIF_DONE)
-    return fd;
+    return AINPARAMS_AIF_ERROR;
 
   while (strncmp((char *) c,"COMM",4))
     {
       if (ainparams_aif_getlong(fd,4, &i) != AINPARAMS_AIF_DONE)
-	return fd;
-      if (ainparams_aif_flushbytes(fd, i)!= AINPARAMS_AIF_DONE)
-	return fd;
+	return AINPARAMS_AIF_ERROR;
+      if (ainparams_aif_flushbytes(fd, i + (i % 2))!= AINPARAMS_AIF_DONE)
+	return AINPARAMS_AIF_ERROR;
       if (ainparams_aif_getbytes(fd, &c[0], 4)!= AINPARAMS_AIF_DONE)
-	return fd;
+	return AINPARAMS_AIF_ERROR;
     }
 
-
   if (ainparams_aif_flushbytes(fd, 4)!= AINPARAMS_AIF_DONE) /* length */
-    return fd;
+    return AINPARAMS_AIF_ERROR;
   if (ainparams_aif_getlong(fd, 2, &i) != AINPARAMS_AIF_DONE)
-    return fd;
+    return AINPARAMS_AIF_ERROR;
   if (i > 0)
     *inchannels = i;
 
   if (ainparams_aif_flushbytes(fd, 4)!= AINPARAMS_AIF_DONE) /* frames */
-    return fd;
+    return AINPARAMS_AIF_ERROR;
   if (ainparams_aif_getlong(fd, 2, &i) != AINPARAMS_AIF_DONE)
-    return fd;
-  if (i != 16)
+    return AINPARAMS_AIF_ERROR;
+  if ((i < 8) || (i > 24))
     {
       fprintf(stderr,"Error: Can't handle %li bit data\n",i);
-      return fd;
+      return AINPARAMS_AIF_ERROR;
     }
   if (ainparams_aif_getslong(fd, 2, &e) != AINPARAMS_AIF_DONE)
-    return fd;
+    return AINPARAMS_AIF_ERROR;
   if (ainparams_aif_getlong(fd, 4, &m) != AINPARAMS_AIF_DONE)
-    return fd;
+    return AINPARAMS_AIF_ERROR;
 
   i = (unsigned long)(0.5+(m*exp(log(2)*(e - 16414.0F))));
   if (i > 0)
     *srate = i;
 
-  return fd;
-
-
+  return AINPARAMS_AIF_DONE;
 }
 
 /*********************************************************/
@@ -1214,10 +1328,17 @@ void ainparams_aif(int * srate, int * inchannels)
     fd = fopen("output.aif","rb");
   
   if (!fd)
-    return;
+    {
+      fprintf(stderr, "Warning: -ain AIF file %s cannot be opened.\n",
+	      ainname ? ainname : "(default name: output.aif)");
+      return;
+    }
 
-  fclose(ainparams_aif_reader(fd, srate, inchannels));
+  if (ainparams_aif_reader(fd, srate, inchannels) == AINPARAMS_AIF_ERROR)
+    fprintf(stderr, "Warning: -ain AIF file %s cannot be parsed.\n",
+	    ainname ? ainname : "(default name: output.aif)");
 
+  fclose(fd);
 }
 
 #undef AINPARAMS_AIF_MATCH  
