@@ -2,25 +2,42 @@
 /*
 #    Sfront, a SAOL to C translator    
 #    This file: aiff audio driver for sfront
-#    Copyright (C) 1999  Regents of the University of California
 #
-#    This program is free software; you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License (Version 2) as
-#    published by the Free Software Foundation.
+# Copyright (c) 1999-2006, Regents of the University of California
+# All rights reserved.
 #
-#    This program is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#    GNU General Public License for more details.
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are
+# met:
 #
-#    You should have received a copy of the GNU General Public License
-#    along with this program; if not, write to the Free Software
-#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#  Redistributions of source code must retain the above copyright
+#  notice, this list of conditions and the following disclaimer.
+#
+#  Redistributions in binary form must reproduce the above copyright
+#  notice, this list of conditions and the following disclaimer in the
+#  documentation and/or other materials provided with the distribution.
+#
+#  Neither the name of the University of California, Berkeley nor the
+#  names of its contributors may be used to endorse or promote products
+#  derived from this software without specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #
 #    Maintainer: John Lazzaro, lazzaro@cs.berkeley.edu
 */
 
 
+
 /****************************************************************/
 /****************************************************************/
 /*             aif file audio driver for sfront                 */ 
@@ -41,10 +58,12 @@ FILE * asyso_fd;     /* output file pointer */
 char * asyso_name;   /* name of file  */        
 long asyso_srate;    /* sampling rate */
 long asyso_channels; /* number of channels */
-long asyso_size;    /* number of samples in a buffer */
+long asyso_size;     /* number of samples in a buffer */
 long asyso_nsamp;    /* total number of shorts written */
+long asyso_bps;      /* number of bytes per sample, 1-3 */
 long asyso_doswap;   /* needs byteswap on write */
-short * asyso_buf;   /* location for output buffer */ 
+float * asyso_buf;   /* output floats from sfront */ 
+unsigned char * asyso_cbuf;  /* output chars to file */
 #endif
 
 #if defined(ASYS_HASINPUT)
@@ -65,10 +84,12 @@ FILE * asysi_fd;     /* input file pointer */
 char * asysi_name;   /* name of file  */        
 long asysi_srate;    /* sampling rate */
 long asysi_channels; /* number of channels */
-long asysi_size;    /* number of samples in a buffer */
 long asysi_nsamp;    /* total number of shorts read */
+long asysi_bytes;    /* number of bytes in a buffer */
+long asysi_bps;      /* number of bytes per sample, 1-3 */
 long asysi_doswap;   /* needs byteswap on read */
-short * asysi_buf;   /* location for input buffer */ 
+unsigned char * asysi_cbuf;  /* buffer of AIFF file bytes */
+float * asysi_buf;   /* float buffer for sfront */ 
 
 #endif
 
@@ -141,8 +162,20 @@ int asyso_setup(long srate, long ochannels, long osize, char * name)
     val = ASYSO_DEFAULTNAME;
   else
     val = name;
-  asyso_name = strcpy((char *) calloc((strlen(val)+1),sizeof(char)), val);
 
+  switch (ASYS_OUTFILE_WORDSIZE) {
+  case ASYS_OUTFILE_WORDSIZE_8BIT: 
+    asyso_bps = 1;
+    break;
+  case ASYS_OUTFILE_WORDSIZE_16BIT:
+    asyso_bps = 2;
+    break;
+  case ASYS_OUTFILE_WORDSIZE_24BIT:
+    asyso_bps = 3;
+    break;
+  }
+
+  asyso_name = strcpy((char *) calloc((strlen(val)+1),sizeof(char)), val);
   asyso_fd = fopen(asyso_name,"wb");
   if (asyso_fd == NULL)
     return ASYS_ERROR;
@@ -155,7 +188,7 @@ int asyso_setup(long srate, long ochannels, long osize, char * name)
   asyso_putlong(18,4);                 /* 18 bytes */
   asyso_putlong(ochannels,2);          /* number of channels */
   asyso_putlong(0,4);                  /* frames, patched later */
-  asyso_putlong(16,2);                 /* 16 bit data */
+  asyso_putlong(8*asyso_bps, 2);       /* bits per sample */
   m = (unsigned long)floor(ldexp(frexp((double)srate, &e),32));
   e += 16382;
   c[0] = e >> 8;
@@ -175,7 +208,24 @@ int asyso_setup(long srate, long ochannels, long osize, char * name)
   asyso_channels = ochannels;
   asyso_size = osize;
   asyso_nsamp = 0;
-  asyso_buf = (short *)calloc(osize, sizeof(short));
+  asyso_cbuf = (unsigned char *) malloc(osize*asyso_bps);
+
+  if (asyso_cbuf == NULL)
+    {
+      fprintf(stderr, "Can't allocate AIFF byte output buffer (%s).\n",
+	      strerror(errno));
+      return ASYS_ERROR;
+    }
+
+  asyso_buf = (float *)calloc(osize, sizeof(float));
+
+  if (asyso_buf == NULL)
+    {
+      fprintf(stderr, "Can't allocate AIFF float output buffer (%s).\n",
+	      strerror(errno));
+      return ASYS_ERROR;
+    }
+
   return ASYS_DONE;
 }
 
@@ -315,7 +365,7 @@ int asysi_setup(long srate, long ichannels, long isize, char * name)
 
 {
   short swaptest = 0x0001;
-  unsigned long i, m;
+  unsigned long i, m, commlen;
   long e, len;
   unsigned char c[4];
   char * val;
@@ -344,12 +394,12 @@ int asysi_setup(long srate, long ichannels, long isize, char * name)
 	{
 	  if (asysi_getlong(4, &i) != ASYS_DONE)
 	    return ASYS_ERROR;
-	  if (asysi_flushbytes(i)!= ASYS_DONE)
+	  if (asysi_flushbytes(i + (i % 2))!= ASYS_DONE)
 	    return ASYS_ERROR;
 	}
       else
 	{
-	  if (asysi_flushbytes(4)!= ASYS_DONE) /* length */
+	  if (asysi_getlong(4, &commlen) != ASYS_DONE)
 	    return ASYS_ERROR;
 	  if (asysi_getlong(2, &i) != ASYS_DONE)
 	    return ASYS_ERROR;
@@ -362,11 +412,12 @@ int asysi_setup(long srate, long ichannels, long isize, char * name)
 	    return ASYS_ERROR;
 	  if (asysi_getlong(2, &i) != ASYS_DONE)
 	    return ASYS_ERROR;
-	  if (i != 16)
+	  if ((i < 8) || (i > 24))
 	    {
 	      fprintf(stderr,"Error: Can't handle %i bit data\n",i);
 	      return ASYS_ERROR;
 	    }
+	  asysi_bps = i/8;
 	  if (asysi_getslong(2, &e) != ASYS_DONE)
 	    return ASYS_ERROR;
 	  if (asysi_getlong(4, &m) != ASYS_DONE)
@@ -377,7 +428,9 @@ int asysi_setup(long srate, long ichannels, long isize, char * name)
 	  if (srate != i)
 	    fprintf(stderr,"Warning: SAOL srate %i mismatches AIFF file srate %i\n",
 		    srate, i);
-	    
+	  if (commlen > 18)
+	    if (asysi_flushbytes(commlen - 18 + (commlen % 2))!= ASYS_DONE) 
+	      return ASYS_ERROR;
 	}
       if (asysi_getbytes(&c[0],4)!= ASYS_DONE)
 	return ASYS_ERROR;
@@ -386,12 +439,29 @@ int asysi_setup(long srate, long ichannels, long isize, char * name)
     return ASYS_ERROR;
   if (asysi_flushbytes(8)!= ASYS_DONE) 
     return ASYS_ERROR;
-  asysi_nsamp = (i - 8)/2;
-  
+
+  asysi_nsamp = (i - 8)/asysi_bps;
   asysi_srate = srate;
   asysi_channels = ichannels;
-  asysi_size = isize;
-  asysi_buf = (short *)malloc(sizeof(short)*isize);
+  asysi_bytes = isize*asysi_bps;
+  asysi_cbuf = (unsigned char *) malloc(asysi_bytes);
+
+  if (asysi_cbuf == NULL)
+    {
+      fprintf(stderr, "Can't allocate AIFF input byte buffer (%s).\n",
+	      strerror(errno));
+      return ASYS_ERROR;
+    }
+
+  asysi_buf = (float *) malloc(sizeof(float)*isize);
+
+  if (asysi_buf == NULL)
+    {
+      fprintf(stderr, "Can't allocate AIFF input float buffer (%s).\n",
+	      strerror(errno));
+      return ASYS_ERROR;
+    }
+
   return ASYS_DONE;
 }
 
@@ -459,11 +529,11 @@ void asyso_shutdown(void)
 {
 
   fseek(asyso_fd, 4, SEEK_SET);
-  asyso_putlong(2*asyso_nsamp+46,4);
-  fseek(asyso_fd,22,SEEK_SET); /* bugfix by Richard Dobson */
-  asyso_putlong(asyso_nsamp /asyso_channels,4);
+  asyso_putlong(asyso_bps*asyso_nsamp+46, 4);
+  fseek(asyso_fd, 22, SEEK_SET); /* bugfix by Richard Dobson */
+  asyso_putlong(asyso_nsamp/asyso_channels, 4);
   fseek(asyso_fd, 16, SEEK_CUR);
-  asyso_putlong(8+2*asyso_nsamp,4);
+  asyso_putlong(8 + asyso_bps*asyso_nsamp, 4);
   fclose(asyso_fd);
 }
 
@@ -552,26 +622,45 @@ int asys_preamble(ASYS_OTYPE * asys_obuf[], long * osize)
 int asys_putbuf(ASYS_OTYPE * asys_obuf[], long * osize)
 
 {
-  unsigned char * buf;
-  unsigned char tmp;
+  float * buf = *asys_obuf;
+  float fval;
+  long val;
   long i = 0;
+  long j = 0;
+
+  switch (asyso_bps) {
+  case 3:
+    while (i < *osize)
+      {
+	fval = ((float)(pow(2, 23) - 1))*buf[i++];
+	val = (long)((fval >= 0.0F) ? (fval + 0.5F) : (fval - 0.5F));
+	asyso_cbuf[j++] = (unsigned char)((val >> 16) & 0x000000FF);
+	asyso_cbuf[j++] = (unsigned char)((val >> 8) & 0x000000FF);
+	asyso_cbuf[j++] = (unsigned char) (val & 0x000000FF);
+      }
+    break;
+  case 2:
+    while (i < *osize)
+      {
+	fval = ((float)(pow(2, 15) - 1))*buf[i++];
+	val = (long)((fval >= 0.0F) ? (fval + 0.5F) : (fval - 0.5F));
+	asyso_cbuf[j++] = (unsigned char)((val >> 8) & 0x000000FF);
+	asyso_cbuf[j++] = (unsigned char) (val & 0x000000FF);
+      }
+    break;
+  case 1:
+    while (i < *osize)
+      {
+	fval = ((float)(pow(2, 7) - 1))*buf[i++];
+	asyso_cbuf[j++] = (unsigned char)
+	  (((char)((fval >= 0.0F) ? (fval + 0.5F) : (fval - 0.5F))));
+      }
+    break;
+  }
+
+  if (rwrite(asyso_cbuf, sizeof(char), j, asyso_fd) != j)
+    return ASYS_ERROR;
 
-  if (asyso_doswap)
-    {
-      buf = (unsigned char *)(*asys_obuf);
-      while (i < 2*(*osize))
-	{
-	  tmp = buf[i+1];
-	  buf[i+1] = buf[i];
-	  buf[i] = tmp;
-	  i += 2;
-	}
-      if (rwrite(buf, sizeof(char), 2*(*osize), asyso_fd) != 2*(*osize))
-	return ASYS_ERROR;
-    }
-  else
-    if (rwrite(*asys_obuf, sizeof(short), *osize, asyso_fd) != *osize)
-      return ASYS_ERROR;
   asyso_nsamp += *osize;
   *osize = asyso_size;
   return ASYS_DONE;
@@ -582,15 +671,14 @@ int asys_putbuf(ASYS_OTYPE * asys_obuf[], long * osize)
 #if defined(ASYS_HASINPUT)
 
 /****************************************************************/
-/*               sends one frame of audio to output             */
+/*               get one frame of audio from input              */
 /****************************************************************/
 
 int asys_getbuf(ASYS_ITYPE * asys_ibuf[], long * isize)
 
 {
-  unsigned char * buf;
-  unsigned char tmp;
   long i = 0;
+  long j = 0;
 
   if (*asys_ibuf == NULL)
     *asys_ibuf = asysi_buf;
@@ -601,18 +689,39 @@ int asys_getbuf(ASYS_ITYPE * asys_ibuf[], long * isize)
       return ASYS_DONE;
     }
 
-  *isize = (long)rread(*asys_ibuf, sizeof(short), asysi_size, asysi_fd);
-  if (asysi_doswap)
-    {
-      buf = (unsigned char *)(*asys_ibuf);
-      while (i < 2*(*isize))
-	{
-	  tmp = buf[i+1];
-	  buf[i+1] = buf[i];
-	  buf[i] = tmp;
-	  i += 2;
-	}
-    }
+  *isize = (long)rread(asysi_cbuf, sizeof(unsigned char), asysi_bytes, asysi_fd);
+
+  switch (asysi_bps) {
+  case 1:                              /* 8-bit  */
+    while (i < *isize)
+      {
+	asysi_buf[i] = ((float)pow(2, -7))*((signed char) asysi_cbuf[i]);
+	i++;
+      }
+    break;
+  case 2:                              /* 9-16 bit */
+    *isize = (*isize) / 2;
+    while (i < *isize)
+      {
+	asysi_buf[i] = ((float)pow(2, -15))*((long)(asysi_cbuf[j+1]) + 
+				    (((long)((char)(asysi_cbuf[j]))) << 8)); 
+	i++;
+	j += 2;
+      }
+    break;
+  case 3:                            /* 17-24 bit */
+    *isize = (*isize) / 3;
+    while (i < *isize)
+      {
+	asysi_buf[i] = ((float)pow(2, -23))*((long)(asysi_cbuf[j+2]) + 
+				    (((long)(asysi_cbuf[j+1])) << 8) + 
+				    (((long)((char) asysi_cbuf[j])) << 16));
+	i++; 
+	j += 3;
+      }
+    break;
+  } 
+
   asysi_nsamp -= *isize;
   return ASYS_DONE;
 }
