@@ -2,25 +2,42 @@
 /*
 #    Sfront, a SAOL to C translator    
 #    This file: Apple CoreAudio driver for sfront
-#    Copyright (C) 2002  Regents of the University of California
 #
-#    This program is free software; you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License (Version 2) as
-#    published by the Free Software Foundation.
+# Copyright (c) 1999-2006, Regents of the University of California
+# All rights reserved.
 #
-#    This program is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#    GNU General Public License for more details.
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are
+# met:
 #
-#    You should have received a copy of the GNU General Public License
-#    along with this program; if not, write to the Free Software
-#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#  Redistributions of source code must retain the above copyright
+#  notice, this list of conditions and the following disclaimer.
+#
+#  Redistributions in binary form must reproduce the above copyright
+#  notice, this list of conditions and the following disclaimer in the
+#  documentation and/or other materials provided with the distribution.
+#
+#  Neither the name of the University of California, Berkeley nor the
+#  names of its contributors may be used to endorse or promote products
+#  derived from this software without specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #
 #    Maintainer: John Lazzaro, lazzaro@cs.berkeley.edu
 */
 
 
+
 /*~~~~~~~~~~~~~~~~~*/
 /* include headers */
 /*_________________*/
@@ -796,7 +813,7 @@ OSStatus asysn_coreaudio_only_input(AudioDeviceID inDevice,
 void asysn_coreaudio_page()
 {
   UInt32 psize, pagetop, addrtop, touchtop, vm_total, err;
-  int statsize = HOST_VM_INFO_COUNT;
+  unsigned int statsize = HOST_VM_INFO_COUNT;
   vm_statistics_data_t vm_stat;
   struct vm_region_basic_info rinfo;
   mach_msg_type_number_t rinfo_count;
