@@ -1,27 +1,43 @@
 
-
 /*
 #    Sfront, a SAOL to C translator    
 #    This file: Code for table generators
-#    Copyright (C) 1999  Regents of the University of California
 #
-#    This program is free software; you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License (Version 2) as
-#    published by the Free Software Foundation.
+# Copyright (c) 1999-2006, Regents of the University of California
+# All rights reserved.
 #
-#    This program is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#    GNU General Public License for more details.
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are
+# met:
 #
-#    You should have received a copy of the GNU General Public License
-#    along with this program; if not, write to the Free Software
-#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#  Redistributions of source code must retain the above copyright
+#  notice, this list of conditions and the following disclaimer.
 #
-#    Primary Author: John Lazzaro, lazzaro@cs.berkeley.edu
+#  Redistributions in binary form must reproduce the above copyright
+#  notice, this list of conditions and the following disclaimer in the
+#  documentation and/or other materials provided with the distribution.
+#
+#  Neither the name of the University of California, Berkeley nor the
+#  names of its contributors may be used to endorse or promote products
+#  derived from this software without specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+#
+#    Maintainer: John Lazzaro, lazzaro@cs.berkeley.edu
 */
 
 
+
 int tgen_init(tableinfo * ti, int pnum, float * p)
 
 {
@@ -32,21 +48,25 @@ int tgen_init(tableinfo * ti, int pnum, float * p)
   ti->start = 0;
   ti->end = 0;
   ti->sr = 0;
-  ti->dmult = 0;
-  ti->lmult = 0;
+  ti->dint = 0;
+  ti->dfrac = 0;
   ti->base = 0;
   ti->stamp = scorebeats;
   if ((pnum < 1) || ((i = ti->len = ROUND(p[0])) < 1))
     epr(0,"control driver","tgen_init","Table length < 1");
   ti->lenf = (float) i;
-  ti->m = 0;
-  ti->n = (i - 1.0F)/i;
-  ti->diff = (i - 1.0F)/i;
+  ti->oconst = ((float) i)*ATIME;
   ti->tend = i - 1;
   ti->t = (float *) calloc(i+1, sizeof(float));
   ti->llmem = 1;
-  return i;
 
+#if INTERP_TYPE == INTERP_SINC  
+  ti->sffl = 1.0F;
+  ti->sfui = 0x00010000;
+  ti->dsincr = SINC_PILEN;
+#endif
+
+  return i;
 }
 
 
@@ -276,8 +296,11 @@ void tgen_destroy(tableinfo * ti, int pnum, float * p)
 
 {  
   ti->len = ti->start = ti->end = ti->tend = 0;
-  ti->lenf = ti->sr = ti->base = ti->m = 0.0F;
-  ti->n = ti->dmult = ti->lmult = ti->diff = 0.0F;
+  ti->lenf = ti->oconst = ti->sr = ti->base = 0.0F;
+  ti->dint = ti->dfrac = 0;
+  ti->sffl = 0;
+  ti->sfui = 0;
+  ti->dsincr = 0;
   if (ti->llmem)
     free(ti->t); 
   ti->llmem = 0;
@@ -647,12 +670,14 @@ void tgen_sample(tableinfo * ti, int pnum, float * p)
 
 {
   int i;
+  double intdummy;
 
   if (pnum < SAMP_DATABLOCK + 1)
     epr(0,"control driver","tgen_sample","Zero-length sample");
 
   ti->len = pnum - SAMP_DATABLOCK;
   ti->lenf = (float)(pnum - SAMP_DATABLOCK);
+  ti->oconst = ti->lenf*ATIME;
 
   if (ti->llmem)
     free(ti->t); 
@@ -663,34 +688,70 @@ void tgen_sample(tableinfo * ti, int pnum, float * p)
   if (p[pnum - SAMP_SR] > 0)
     {
       ti->sr = p[pnum - SAMP_SR];
-      ti->dmult = ti->sr*ATIME/ti->lenf;
+      if (p[pnum - SAMP_SR] == ARATE)
+	{
+	  ti->dint = 1;
+	  ti->dfrac = 0;
+	}
+      else
+	{
+	  ti->dfrac = 4294967296.0*
+	    modf(((double)p[pnum - SAMP_SR])/((double)ARATE), &intdummy);
+	  ti->dint = intdummy;
+	}
     }
   else
     {
       ti->sr = ARATE;
-      ti->dmult = 1.0F/ti->lenf;
+      ti->dint = 1;
+      ti->dfrac = 0;
     }
 
+#if (INTERP_TYPE == INTERP_SINC)
+
+  if (p[pnum - SAMP_SR] > 0)
+    {
+      if (ARATE >= p[pnum - SAMP_SR])
+	{
+	  ti->sffl = 1.0F;
+	  ti->sfui = 0x00010000;
+	  ti->dsincr = SINC_PILEN;
+	}
+      else
+	{
+	  if ((ARATE*SINC_UPMAX) >= p[pnum - SAMP_SR])
+	    ti->sffl = (ARATE/ti->sr);
+	  else
+	    ti->sffl = (1.0F/SINC_UPMAX);
+	  ti->sfui = ((float)(pow(2,16)))*ti->sffl + 0.5F;
+	  ti->dsincr = (SINC_PILEN*ti->sfui) >> 16;
+	}
+    }
+  else
+    {
+      ti->sffl = 1.0F;
+      ti->sfui = 0x00010000;
+      ti->dsincr = SINC_PILEN;
+    }
+
+#endif
+
   if (p[pnum - SAMP_BASEFREQ] > 0)
     {
       ti->base = p[pnum - SAMP_BASEFREQ];
-      ti->lmult = ti->dmult / p[pnum - SAMP_BASEFREQ];
     }
   else
     {  
       ti->base = 0;
-      ti->lmult = (1.0/ti->lenf)*ATIME;
     }
 
   if (p[pnum - SAMP_LOOPSTART] > 0)
     {
       ti->start = p[pnum - SAMP_LOOPSTART];
-      ti->m = (float)(p[pnum - SAMP_LOOPSTART])/ ti->lenf;
     }
   else
     {
       ti->start = 0;
-      ti->m = 0;
     }
 
   if (p[pnum - SAMP_LOOPEND] > 0)
@@ -699,19 +760,14 @@ void tgen_sample(tableinfo * ti, int pnum, float * p)
       if (ti->end==0)
 	{
 	  ti->tend = ti->len - 1;
-	  ti->n = 1.0F - (1.0F/ti->len);
 	}
-      else
-	ti->n = (float)(p[pnum - SAMP_LOOPEND]/ti->lenf);
     }
   else
     {
       ti->end = 0;
-      ti->n = (i - 1.0F)/i;
       ti->tend = i - 1;
     }
   ti->llmem = (int)(p[pnum - SAMP_LLMEM]);
-  ti->diff = ti->n - ti->m;
   ti->t[ti->len] = ti->t[0];
 }
 
