@@ -3,7 +3,7 @@
 #    Sfront, a SAOL to C translator    
 #    This file: Network library -- constants and externs
 #
-# Copyright (c) 2000, Regents of the University of California
+# Copyright (c) 2000-2006, Regents of the University of California
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -97,6 +97,12 @@
 
 #define NSYS_LATENOTES_DEBUG NSYS_LATENOTES_DEBUG_OFF
 
+#define NSYS_LOSSMODEL_OFF       0
+#define NSYS_LOSSMODEL_ON        1
+
+#define NSYS_LOSSMODEL_LOSSRATE   0.2F      /* between 0.0F and 1.0F */
+#define NSYS_LOSSMODEL NSYS_LOSSMODEL_OFF
+
 #define NSYS_NETOUT_CHAPTERE_OFF   0        /* don't send Chapter E */
 #define NSYS_NETOUT_CHAPTERE_ON    1        /* send Chapter E       */
 
@@ -646,6 +652,10 @@
 
 #define NSYS_SM_JH_CHANMASK 0x0F  /* to mask channel number  */
 
+#define NSYS_SM_JH_SETH  16  /* | to set H header bit   */
+#define NSYS_SM_JH_CLRH 239  /* & to clear A header bit */
+#define NSYS_SM_JH_CHKH  16  /* & to check H header bit   */
+
 #define NSYS_SM_JH_SETA  32  /* | to set A header bit   */
 #define NSYS_SM_JH_CLRA 223  /* & to clear A header bit */
 #define NSYS_SM_JH_CHKA  32  /* & to check A header bit   */
@@ -695,6 +705,10 @@
 #define NSYS_SM_CH_LOC_LEN   0   /* location of 10-bit length */
 #define NSYS_SM_CH_LOC_TOC   2   /* location of table of contents */
 
+#define NSYS_SM_CH_SETH   4      /* | to set H header bit      */
+#define NSYS_SM_CH_CLRH 251      /* & to clear H header bit    */ 
+#define NSYS_SM_CH_CHKH   4      /* & to check H header bit    */ 
+
 #define NSYS_SM_CH_CHANMASK 0x78   /* to extract channel number  */
 #define NSYS_SM_CH_LENMASK  0x03FF /* to extract channel length  */
 
@@ -836,9 +850,17 @@
 #define NSYS_SM_CM_CLRX        127     /* & to clear X log bit */
 #define NSYS_SM_CM_CHKX        128     /* & to check X log bit */
 
-#define NSYS_SM_CM_SETG         64     /* | to set G log bit   */
-#define NSYS_SM_CM_CLRG        191     /* & to clear G log bit */
-#define NSYS_SM_CM_CHKG         64     /* & to check G log bit */
+#define NSYS_SM_CM_BUTTON_SETG  128    /* | to set G log bit   */
+#define NSYS_SM_CM_BUTTON_CLRG  127    /* & to clear G log bit */
+#define NSYS_SM_CM_BUTTON_CHKG  128    /* & to check G log bit */
+
+#define NSYS_SM_CM_BUTTON_SETX  64     /* | to set X log bit   */
+#define NSYS_SM_CM_BUTTON_CLRX  191    /* & to clear X log bit */
+#define NSYS_SM_CM_BUTTON_CHKX  64     /* & to check X log bit */
+
+#define NSYS_SM_CM_BUTTON_SETR  64     /* | to set R log bit   */
+#define NSYS_SM_CM_BUTTON_CLRR  191    /* & to clear R log bit */
+#define NSYS_SM_CM_BUTTON_CHKR  64     /* & to check R log bit */
 
 #define NSYS_SM_CM_COUNT_SIZE    1
 
@@ -1068,7 +1090,7 @@
 #define NSYS_SM_CD_COMMON_LOC_FIELDS  2   /* Start of fields    */  
 
 #define NSYS_SM_CD_COMMON_TOC_SIZE    1    /* TOC size         */  
-#define NSYS_SM_CD_COMMON_LENGTH_SIZE 1    /* Log Length Size  */
+#define NSYS_SM_CD_COMMON_LENGTH_SIZE 1    /* Log Length Size (+2 bits) */
 #define NSYS_SM_CD_COMMON_COUNT_SIZE  1    /* Count Field Size */
 
 #define NSYS_SM_CD_COMMON_TOC_SETC     64    /* Count field */
@@ -1083,10 +1105,12 @@
 #define NSYS_SM_CD_COMMON_TOC_CLRL    239    
 #define NSYS_SM_CD_COMMON_TOC_CHKL     16    
 
-#define NSYS_SM_CD_COMMON_DSZ_MASK   0x03    /* bit-mask for DSZ field */
+#define NSYS_SM_CD_COMMON_DSZ_MASK    0x0C    /* bit-mask for DSZ field */
+#define NSYS_SM_CD_COMMON_LENMSB_MASK 0x03    /* bit-mask for LEN MSBs */
 
 #define NSYS_SM_CD_COMMON_SIZE          5      /* implementation-specific */
 
+
 /*  undefined System Real-Time log */
 
 #define NSYS_SM_CD_REALTIME_LOC_TOC     0   /* Table of Contents  */ 
@@ -1574,8 +1598,8 @@ typedef struct nsys_netout_jsend_state {
   unsigned long chapterw_seqnum;    
   unsigned long chaptern_seqnum;    
   unsigned long chaptern_seqarray[NSYS_SM_CN_ARRAYSIZE];
-  unsigned long chaptere_seqnum;    
-  unsigned long chaptere_seqarray[NSYS_SM_CE_ARRAYSIZE];
+  unsigned long chaptere_seqnum;
+  unsigned long chapterer_seqarray[NSYS_SM_CE_ARRAYSIZE];
   unsigned long chapterev_seqarray[NSYS_SM_CE_ARRAYSIZE];
   unsigned long chaptert_seqnum;    
   unsigned long chaptera_seqnum;
@@ -1621,10 +1645,9 @@ typedef struct nsys_netout_jsend_state {
   /* sender state for chapter e */
   
   unsigned char chaptere_ref[NSYS_SM_CE_ARRAYSIZE];     /* reference count   */
-  unsigned long chaptere_logptr[NSYS_SM_CE_ARRAYSIZE];  /* ref log positions */
+  unsigned long chapterer_logptr[NSYS_SM_CE_ARRAYSIZE];  /* ref log positions */
   unsigned long chapterev_logptr[NSYS_SM_CE_ARRAYSIZE]; /* off vel positions */
   int chaptere_sset;                                    /* S bit indicator */
-  int chapterev_sset;                                   /* S bit indicator */
   
   /* sender state for chapter a */
   
@@ -1639,7 +1662,6 @@ typedef struct nsys_netout_jsend_state {
   int chaptern_size;  /* chapter n */
   int chapterb_size;  /* chapter n - bitfields */
   int chaptere_size;  /* chapter e - header + ref log array */
-  int chapterev_size; /* chapter e - off velocity log array */
   int chaptera_size;  /* chapter a */
   
   /* holds current packet bytes */
@@ -1652,8 +1674,7 @@ typedef struct nsys_netout_jsend_state {
   unsigned char chapterw[NSYS_SM_CW_SIZE];        /* chapter w  */
   unsigned char chaptern[NSYS_SM_CN_SIZE];        /* chapter n  */
   unsigned char chapterb[NSYS_SM_CB_SIZE];        /* chapter n - bfields */
-  unsigned char chaptere[NSYS_SM_CE_SIZE];        /* chapter e - ref logs */
-  unsigned char chapterev[NSYS_SM_CE_SIZE];       /* chapter e - vel logs */
+  unsigned char chaptere[NSYS_SM_CE_SIZE];        /* chapter e */
   unsigned char chaptert[NSYS_SM_CT_SIZE];        /* chapter t  */
   unsigned char chaptera[NSYS_SM_CA_SIZE];        /* chapter a  */
   
@@ -2160,18 +2181,12 @@ extern void nsys_netout_journal_addnoteon(nsys_netout_jsend_state * jsend,
 					  unsigned char ndata, 
 					  unsigned char vdata);
 
-extern void nsys_netout_journal_onrefcount(nsys_netout_jsend_state * jsend, 
-					   unsigned char ndata);
+extern void nsys_netout_journal_addnoteon_extras(nsys_netout_jsend_state * jsend, 
+						 unsigned char ndata);
 
-extern void nsys_netout_journal_offrefcount(nsys_netout_jsend_state * jsend, 
-					    unsigned char ndata);
-
-extern void nsys_netout_journal_onrelease(nsys_netout_jsend_state * jsend, 
-					  unsigned char ndata);
-
-extern void nsys_netout_journal_offrelease(nsys_netout_jsend_state * jsend, 
-					  unsigned char ndata,
-					  unsigned char vdata);
+extern void nsys_netout_journal_addnoteoff_extras(nsys_netout_jsend_state * jsend, 
+						  unsigned char ndata,
+						  unsigned char vdata);
 
 extern void nsys_netout_journal_addctouch(nsys_netout_jsend_state * jsend, 
 					  unsigned char ndata);
