@@ -3,7 +3,7 @@
 #    Sfront, a SAOL to C translator    
 #    This file: Network library -- receiver journal functions
 #
-# Copyright (c) 2000, Regents of the University of California
+# Copyright (c) 2000-2006, Regents of the University of California
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -183,7 +183,7 @@ int nsys_netin_journal_recovery(nsys_source * sptr, int rtpcode,
 			      NSYS_SM_CD_COMMON_LENGTH_SIZE)) < 0)
 		return NSYS_JOURNAL_CORRUPTED;
 
-	      loglen = ps[1];
+	      loglen = ps[1] + ((ps[0] & NSYS_SM_CD_COMMON_LENMSB_MASK) << 8);
 
 	      if (((!(ps[0] & NSYS_SM_CHKS)) || many) &&
 		  nsys_netin_jrec_scj(sptr, ps, jrecvsys, buff, fill, size))
@@ -200,7 +200,7 @@ int nsys_netin_journal_recovery(nsys_source * sptr, int rtpcode,
 			      NSYS_SM_CD_COMMON_LENGTH_SIZE)) < 0)
 		return NSYS_JOURNAL_CORRUPTED;
 
-	      loglen = ps[1];
+	      loglen = ps[1] + ((ps[0] & NSYS_SM_CD_COMMON_LENMSB_MASK) << 8);
 
 	      if (((!(ps[0] & NSYS_SM_CHKS)) || many) &&
 		  nsys_netin_jrec_sck(sptr, ps, jrecvsys, buff, fill, size))
@@ -604,17 +604,32 @@ void nsys_netin_journal_trackstate(nsys_source * sptr, unsigned char cmd,
 	  case CSYS_MIDI_CC_MONOMODE:
 	  case CSYS_MIDI_CC_POLYMODE:
 	    memset(jrecv->chaptern_ref, 0, NSYS_SM_CN_ARRAYSIZE);
-	    memset(jrecv->chaptera_pressure, 0, NSYS_SM_CA_ARRAYSIZE);
 	    jrecv->chaptert_pressure = 0;
 	    break;
 	  case CSYS_MIDI_CC_RESETALLCONTROL:
-	    memset(jrecv->chapterc_value, 0, CSYS_MIDI_CC_ALLSOUNDOFF);
-	    memset(jrecv->chapterm_value_lsb, 0, NSYS_SM_CM_ARRAYSIZE);
-	    memset(jrecv->chapterm_value_msb, 0, NSYS_SM_CM_ARRAYSIZE);
-	    memset(jrecv->chapterm_cbutton, 0, sizeof(short)*NSYS_SM_CM_ARRAYSIZE);
+
+	    /* Chapter C -- update pedal count */
+
+	    jrecv->chapterc_value[CSYS_MIDI_CC_SUSTAIN] = NSYS_SM_RV_SETF |
+	      (((jrecv->chapterc_value[CSYS_MIDI_CC_SUSTAIN] & NSYS_SM_CC_ALTMOD) + 1)
+	       & NSYS_SM_CC_ALTMOD);
+
+	    /* Clear parameter transaction system */
+
 	    jrecv->chapterm_nrpn_msb = jrecv->chapterm_nrpn_lsb = 0;
 	    jrecv->chapterm_rpn_msb = jrecv->chapterm_rpn_lsb = 0;
 	    jrecv->chapterm_state = NSYS_SM_CM_STATE_OFF;
+
+	    /* Clear parameter system C-BUTTON counts */
+
+	    memset(jrecv->chapterm_cbutton, 0, sizeof(short)*NSYS_SM_CM_ARRAYSIZE);
+
+	    /* C-active Chapters:  W, T, and A */
+
+	    jrecv->chapterw_first = 0;
+	    jrecv->chapterw_second = 0;
+	    memset(jrecv->chaptera_pressure, 0, NSYS_SM_CA_ARRAYSIZE);
+	    jrecv->chaptert_pressure = 0;
 	    break;
 	  }
 	  break;
@@ -1027,12 +1042,33 @@ int nsys_netin_jrec_control(nsys_source * sptr, unsigned char * p,
 		    case CSYS_MIDI_CC_MONOMODE:
 		    case CSYS_MIDI_CC_POLYMODE:
 		      memset(jrecv->chaptern_ref, 0, NSYS_SM_CN_ARRAYSIZE);
-		      memset(jrecv->chaptera_pressure, 0, NSYS_SM_CA_ARRAYSIZE);
 		      jrecv->chaptert_pressure = 0;
 		      break;
 		    case CSYS_MIDI_CC_RESETALLCONTROL:
-		      memset(jrecv->chapterc_value, 0, 
-			     CSYS_MIDI_CC_ALLSOUNDOFF);
+
+		      /* Chapter C -- update pedal count */
+		      
+		      jrecv->chapterc_value[CSYS_MIDI_CC_SUSTAIN] = NSYS_SM_RV_SETF |
+			(((jrecv->chapterc_value[CSYS_MIDI_CC_SUSTAIN] 
+			   & NSYS_SM_CC_ALTMOD) + 1) & NSYS_SM_CC_ALTMOD);
+
+		      /* Clear parameter transaction system */
+		      
+		      jrecv->chapterm_nrpn_msb = jrecv->chapterm_nrpn_lsb = 0;
+		      jrecv->chapterm_rpn_msb = jrecv->chapterm_rpn_lsb = 0;
+		      jrecv->chapterm_state = NSYS_SM_CM_STATE_OFF;
+
+		      /* Clear parameter system C-BUTTON counts */
+
+		      memset(jrecv->chapterm_cbutton, 0, 
+			     sizeof(short)*NSYS_SM_CM_ARRAYSIZE);
+
+		      /* C-active Chapters:  W, T, and A */
+		      
+		      jrecv->chapterw_first = 0;
+		      jrecv->chapterw_second = 0;
+		      memset(jrecv->chaptera_pressure, 0, NSYS_SM_CA_ARRAYSIZE);
+		      jrecv->chaptert_pressure = 0;
 		      break;
 		    }
 		  }
@@ -1218,9 +1254,9 @@ int nsys_netin_jrec_param(nsys_source * sptr, unsigned char * p,
 	{
 	  if ((i + 1) < paramlen)
 	    {
-	      cbutton = p[i] & (NSYS_SM_CM_CLRX & NSYS_SM_CM_CLRG);
+	      cbutton = p[i] & (NSYS_SM_CM_BUTTON_CLRX & NSYS_SM_CM_BUTTON_CLRG);
 	      cbutton = (cbutton << 8) + p[i+1];
-	      if (!(p[i] & NSYS_SM_CM_CHKG))
+	      if (p[i] & NSYS_SM_CM_BUTTON_CHKG)
 		cbutton = - cbutton;
 	      i += NSYS_SM_CM_CBUTTON_SIZE;
 	    }
