@@ -3,7 +3,7 @@
 #    Sfront, a SAOL to C translator    
 #    This file: Network library -- RTP functions
 #
-# Copyright (c) 2000, Regents of the University of California
+# Copyright (c) 2000-2006, Regents of the University of California
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -1102,7 +1102,7 @@ void nsys_netout_rtcp_initsdes(unsigned char * p, int len)
 
   p += NSYS_RTCPLEN_SDES_ITEMHDR;
 
-  strcpy(p, nsys_cname);
+  strcpy((char *) p, nsys_cname);
 
 }
 
