@@ -3,7 +3,7 @@
 #    Sfront, a SAOL to C translator    
 #    This file: Network library -- RTP functions
 #
-# Copyright (c) 2000, Regents of the University of California
+# Copyright (c) 2000-2006, Regents of the University of California
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -211,16 +211,18 @@ int nsys_setup(int block)
 	}
     }
 
-  nsys_md5(nsys_keydigest, nsys_sessionkey, strlen(nsys_sessionkey));
+  nsys_md5(nsys_keydigest, (unsigned char *) nsys_sessionkey, 
+	   strlen(nsys_sessionkey));
 
   for (i = 0; i < NSYS_MD5_ROUNDS; i++)
     nsys_md5(nsys_keydigest, nsys_keydigest, NSYS_MD5_LENGTH);
 
   if (nsys_msession)
-    nsys_md5(session_digest, nsys_sessionname, strlen(nsys_sessionname));
+    nsys_md5(session_digest, (unsigned char *) nsys_sessionname, 
+	     strlen(nsys_sessionname));
   else
-    nsys_hmac_md5(nsys_sessionname, strlen(nsys_sessionname), nsys_keydigest,
-		  session_digest);
+    nsys_hmac_md5((unsigned char *) nsys_sessionname, strlen(nsys_sessionname), 
+		   nsys_keydigest, session_digest);
 
   nsys_digest_base64(nsys_session_base64, session_digest);
 
