@@ -3,7 +3,7 @@
 #    Sfront, a SAOL to C translator    
 #    This file: Sfront-specific network functions
 #
-# Copyright (c) 2000, Regents of the University of California
+# Copyright (c) 2000-2006, Regents of the University of California
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -60,7 +60,7 @@ int nsys_newdata(void)
   unsigned char cmd, oldcmd;
   unsigned char ndata = 0;
   unsigned char vdata = 0;
-  int fromlen = sizeof(struct sockaddr);
+  unsigned int fromlen = sizeof(struct sockaddr);
   int retry = 0;
   int has_ts, dsize, rtpcode, jcode, fec, sysidx, skip;
   int nopacket = 1;
@@ -534,7 +534,7 @@ int nsys_newdata(void)
 
 	  packet[len] = '\0';  /* null-terminate string */
 
-	  if (sscanf(packet, "SIP/2.0 %hu", &status) == 1)
+	  if (sscanf((char *) packet, "SIP/2.0 %hu", &status) == 1)
 	    nsys_netin_reply(fd, &ipaddr, packet, status);
 	  else
 	    if (packet[NSYS_RTPLOC_BYTE1] == 'I')
@@ -842,6 +842,14 @@ void nsys_midisend(unsigned char cmd, unsigned char ndata,
   nsys_sent_octets += (size + 1 + jsize + NSYS_RTPSIZE_DIGEST);
   size += (NSYS_RTPLEN_HDR + 1 + jsize + NSYS_RTPSIZE_DIGEST);     
 
+  /*********************************************************/
+  /* if loss model is active, don't always send the packet */
+  /*********************************************************/
+
+  if (NSYS_LOSSMODEL == NSYS_LOSSMODEL_ON)
+    if (rand() > (int)((1.0F - NSYS_LOSSMODEL_LOSSRATE)*RAND_MAX))
+      return;
+
   /***************************/
   /* send packet to everyone */
   /***************************/
