@@ -3,7 +3,7 @@
 #    Sfront, a SAOL to C translator    
 #    This file: Network library -- SIP functions
 #
-# Copyright (c) 2000, Regents of the University of California
+# Copyright (c) 2000-2006, Regents of the University of California
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -181,7 +181,7 @@ void nsys_netin_reply(int fd, struct sockaddr_in * addr,
       if (fd == nsys_rtp_fd)
 	{
 	  nsys_rtp_cseq++;
-	  invite = nsys_rtp_invite;
+	  invite = (char *) nsys_rtp_invite;
 
 	  nsys_rtcp_ex |=  NSYS_RTCPEX_RTPSIP;
 	  
@@ -195,7 +195,7 @@ void nsys_netin_reply(int fd, struct sockaddr_in * addr,
       else
 	{
 	  nsys_rtcp_cseq++;
-	  invite = nsys_rtcp_invite;
+	  invite = (char *) nsys_rtcp_invite;
 
 	  nsys_rtcp_ex |=  NSYS_RTCPEX_RTCPSIP;
 	  
@@ -207,9 +207,9 @@ void nsys_netin_reply(int fd, struct sockaddr_in * addr,
 	  nsys_createsip(nsys_rtcp_fd, "INFO", nsys_rtcp_info, NULL, NULL); 
 	}
 
-      nsys_createsip(fd, "INVITE", invite, natline[0] ? natline : NULL,
-		     nonce[0] ? nonce : NULL);
-      nsys_sendsip(fd, NULL, invite);
+      nsys_createsip(fd, "INVITE", (unsigned char *) invite, 
+		     natline[0] ? natline : NULL, nonce[0] ? nonce : NULL);
+      nsys_sendsip(fd, NULL, (unsigned char *) invite);
 
       return;
     }
@@ -310,7 +310,7 @@ void nsys_netin_invite(int fd, struct sockaddr_in * addr,
 
   while (state == NSYS_NETIN_SDPSTATE)
     {
-      len = sscanf(packet, "%1023[^\n]\n", line);
+      len = sscanf((char *) packet, "%1023[^\n]\n", line);
       if (len && (len != EOF))
 	{
 	  switch (line[0]) {
@@ -318,7 +318,7 @@ void nsys_netin_invite(int fd, struct sockaddr_in * addr,
 	    break;
 	  case 's':
 	    if ((sscanf(line, "s = %24s", sname) != 1) || 
-		(strcmp(nsys_session_base64, sname)))
+		(strcmp((char *) nsys_session_base64, sname)))
 	      state = NSYS_NETIN_ERRSTATE;
 	    else
 	      has_session = 1;
@@ -576,7 +576,7 @@ void nsys_createsip(int fd, char * method, unsigned char * sip,
   /* create SIP headers */
   /**********************/
 
-  body = snprintf(sip, NSYS_UDPMAXSIZE+1,
+  body = snprintf((char *) sip, NSYS_UDPMAXSIZE+1,
 		  "%s sip:%s@%s:%hu SIP/2.0\n"     /* INVITE or BYE method */
 		  "i: %lu\n"                       /* Call-ID              */
 		  "v: SIP/2.0/UDP %s:%hu\n"        /* Via                  */
@@ -637,7 +637,7 @@ void nsys_createsip(int fd, char * method, unsigned char * sip,
 
   if (nonce)
     {
-      body += snprintf(&(sip[body]), NSYS_UDPMAXSIZE+1-body,
+      body += snprintf((char *) &(sip[body]), NSYS_UDPMAXSIZE+1-body,
 		       "Authorization: SignSDP nonce=\"%s\", "
 		       "digest=\"012345678901234567890A==\"\n",
 		       nonce);
@@ -657,7 +657,7 @@ void nsys_createsip(int fd, char * method, unsigned char * sip,
   for (j = i = 0; i < NSYS_RTP_PAYSIZE; i++)
     j += sprintf(&(media[j]), "%hhu ", nsys_payload_types[i].ptype);
 
-  len = body + snprintf(&(sip[body]), NSYS_UDPMAXSIZE+1-body,
+  len = body + snprintf((char *) &(sip[body]), NSYS_UDPMAXSIZE+1-body,
 		 "\n"
 		 "v=0\n"
 		 "o=%s %lu 0 IN IP4 %s\n"          /* CNAME, SSRC, IP    */
@@ -712,8 +712,8 @@ void nsys_createsip(int fd, char * method, unsigned char * sip,
       /* append rtpmap line */
       /**********************/
 
-      len += snprintf(&(sip[len]), NSYS_UDPMAXSIZE+1 - len, 
-		      "a=rtpmap: %hhu %s/%i\n", 
+      len += snprintf((char *) &(sip[len]), NSYS_UDPMAXSIZE+1 - len, 
+		      "a=rtpmap:%hhu %s/%i\n", 
 		      nsys_payload_types[i].ptype, 
 		      nsys_payload_types[i].name, 
 		      nsys_payload_types[i].srate);
@@ -727,41 +727,30 @@ void nsys_createsip(int fd, char * method, unsigned char * sip,
 
       switch (nsys_payload_types[i].pindex) {
       case NSYS_MPEG4_PINDEX:
-	len += snprintf(&(sip[len]), NSYS_UDPMAXSIZE+1 - len,
+	len += snprintf((char *) &(sip[len]), NSYS_UDPMAXSIZE+1 - len,
 
-	"a=fmtp: %hhu streamtype=5; mode=rtp-midi; profile-level-id=12;\n"
-	"a=fmtp: %hhu config=\"\"; render=synthetic; rinit=audio/asc; \n"
+	"a=fmtp:%hhu streamtype=5; mode=rtp-midi; profile-level-id=12;"
+	" config=\"\"; render=synthetic; subrender=default; rinit=audio/asc;"
+	" tsmode=buffer; octpos=last; mperiod=%lu;"
+	" rtp_ptime=0; rtp_maxptime=0; cm_unused=DEFMQVX;%s"
+	" guardtime=%lu\n",
 
-	"a=fmtp: %hhu tsmode=buffer; octpos=last; mperiod=%lu;\n"
-	"a=fmtp: %hhu ptime=0; maxptime=0; ch_unused=DEFMQVX;%s\n"
+	nsys_payload_types[i].ptype,
 
-	"a=fmtp: %hhu guardtime=%lu;\n",
+	/* tsmode line */
 
-			/* streamtype line */
+	(unsigned long) ACYCLE,
 
-			nsys_payload_types[i].ptype,
+	/* rtp_ptime line */
 
-			/* config line */
+	(nsys_feclevel != NSYS_SM_FEC_NONE) 
+	? "" : " j_sec=none;",
 
-			nsys_payload_types[i].ptype,
+	/* guardtime line */
 
-			/* tsmode line */
+	(unsigned long)(NSYS_SM_GUARD_MAXTIME*ARATE + 0.5F)
 
-			nsys_payload_types[i].ptype,
-			(unsigned long) ACYCLE,
-
-			/* ptime line */
-
-			nsys_payload_types[i].ptype,
-			(nsys_feclevel != NSYS_SM_FEC_NONE) 
-			? "" : " j_sec=none;",
-
-			/* guardtime line */
-
-			nsys_payload_types[i].ptype,
-			(unsigned long)(NSYS_SM_GUARD_MAXTIME*ARATE + 0.5F)
-
-			);
+	);
 
 	if ((len > NSYS_UDPMAXSIZE))
 	  NSYS_ERROR_TERMINATE("SIP INVITE packet creation error [5]");
@@ -814,45 +803,45 @@ unsigned char * nsys_netin_readmethod(int fd, struct sockaddr_in * addr,
   /* extract Call-ID, In-Reply-To, Nonce and Digest from SIP */
   /***********************************************************/
 
-  while ((num = sscanf(packet, "%[^\n]\n", line)) && (num != EOF))
+  while ((num = sscanf((char *) packet, "%[^\n]\n", line)) && (num != EOF))
     {
       if (first)
 	{
 	  first = 0;
-	  found_invite = (line == ((unsigned char *) strstr(line,"INVITE")));
+	  found_invite = (line == ((unsigned char *)strstr((char *)line,"INVITE")));
 
 	  if ((len += strlen("SIP/2.0 200\n")) < NSYS_UDPMAXSIZE+1)
-	    strcpy(reply, "SIP/2.0 200\n");
+	    strcpy((char *) reply, "SIP/2.0 200\n");
 	  else
 	    overflow = 1;
 	}
       else
 	{
-	  if ((len += strlen(line) + 1) < NSYS_UDPMAXSIZE+1)
+	  if ((len += strlen((char *) line) + 1) < NSYS_UDPMAXSIZE+1)
 	    {
-	      strcat(reply, line);
-	      strcat(reply, "\n");
+	      strcat((char *) reply, (char *) line);
+	      strcat((char *) reply, "\n");
 	    }
 	  else
 	    overflow = 1;
 
 	  if ((line[0] == 'i') || (line[0] == 'C'))
-	    found_callid |= ((sscanf(line, "i : %lu", &callid) == 1) || 
-			     (sscanf(line, "Call-ID : %lu", &callid) == 1));
+	    found_callid |= ((sscanf((char *) line, "i : %lu", &callid) == 1) || 
+			     (sscanf((char *) line, "Call-ID : %lu", &callid) == 1));
 	  if (line[0] == 'I')
-	    found_reply |= (sscanf(line, "In-Reply-To : %lu",&inreplyto) == 1);
+	    found_reply |= (sscanf((char *) line, "In-Reply-To : %lu",&inreplyto) == 1);
 
 	  if (line[0] == 'A')
-	    found_auth |= ((sscanf(line, "Authorization : SignSDP " 
+	    found_auth |= ((sscanf((char *) line, "Authorization : SignSDP " 
 				   "nonce = \"%24[^\"]\" , "
 				   "digest = \"%24[^\"]\" , ",
 				   nonce, digest) == 2) || 
-			   (sscanf(line, "Authorization : SignSDP " 
+			   (sscanf((char *) line, "Authorization : SignSDP " 
 				   "digest = \"%24[^\"]\" , "
 				   "nonce = \"%24[^\"]\" , ",
 				   digest, nonce) == 2));
 	}
-      packet += strlen(line) + 1;
+      packet += strlen((char *) line) + 1;
     }
 
   ret = (num != EOF) ? packet + 1 : NULL;
@@ -868,13 +857,13 @@ unsigned char * nsys_netin_readmethod(int fd, struct sockaddr_in * addr,
       nsys_hmac_md5(text, 14, nsys_keydigest, local_binary);
       nsys_digest_base64(local_base64, local_binary);
 
-      if ((!found_auth) || strcmp(local_base64, nonce))
+      if ((!found_auth) || strcmp((char *) local_base64, (char *) nonce))
 	{
-	  sprintf(line, "WWW-Authenticate: SignSDP nonce =\"%s\"\n",
+	  sprintf((char *) line, "WWW-Authenticate: SignSDP nonce =\"%s\"\n",
 		  local_base64);
 
-	  if ((len += strlen(line)) < NSYS_UDPMAXSIZE+1)
-	    strcat(reply, line);
+	  if ((len += strlen((char *)line)) < NSYS_UDPMAXSIZE+1)
+	    strcat((char *) reply, (char *) line);
 	  else
 	    overflow = 1;
 
@@ -887,9 +876,9 @@ unsigned char * nsys_netin_readmethod(int fd, struct sockaddr_in * addr,
 
   if (ret && (nsys_msession == 0))
     {
-      nsys_hmac_md5(ret, strlen(ret), nsys_keydigest, local_binary);
+      nsys_hmac_md5(ret, strlen((char *) ret), nsys_keydigest, local_binary);
       nsys_digest_base64(local_base64, local_binary);
-      if (strcmp(digest, local_base64))
+      if (strcmp((char *) digest, (char *) local_base64))
 	ret = NULL;
     }
 
@@ -921,7 +910,7 @@ void nsys_netin_ack(int fd, unsigned char * reply,
   /* create top of ACK header */
   /****************************/
 
-  len = snprintf(reply, NSYS_UDPMAXSIZE+1, 
+  len = snprintf((char *) reply, NSYS_UDPMAXSIZE+1, 
 	  
 		 "ACK sip:%s@%s:%hu SIP/2.0\n"
 		 "v: SIP/2.0/UDP %s:%hu\n"        /* Via                  */
@@ -986,7 +975,7 @@ int nsys_netin_replyparse(int fd, unsigned char * packet,
   natline[0] = '\0';
   nonce[0] = '\0';
 
-  while (sscanf(packet,"%[^\n]\n",line) == 1)
+  while (sscanf((char *) packet,"%[^\n]\n",line) == 1)
     {
       packet += strlen(line) + 1;
 
@@ -1044,7 +1033,7 @@ void nsys_netin_redirect(int fd, unsigned char * packet,
 
   /* find first contact address, extract IP and port */
 
-  while (sscanf(packet,"%[^\n]\n",line) == 1)
+  while (sscanf((char *) packet,"%[^\n]\n",line) == 1)
     {
       packet += strlen(line) + 1;
       if ((found = (sscanf(line,"m:%*[^@]@%15[^:]:%hu", ip, &port) == 2)))
@@ -1074,7 +1063,7 @@ void nsys_netin_redirect(int fd, unsigned char * packet,
 
   if (fd == nsys_rtp_fd)
     {
-      invite = nsys_rtp_invite;
+      invite = (char *) nsys_rtp_invite;
 
       nsys_rtcp_ex &= ~NSYS_RTCPEX_RTPNEXT;
       nsys_rtcp_ex |=  NSYS_RTCPEX_RTPSIP;
@@ -1083,7 +1072,7 @@ void nsys_netin_redirect(int fd, unsigned char * packet,
     }
   else
     {
-      invite = nsys_rtcp_invite;
+      invite = (char *) nsys_rtcp_invite;
 
       nsys_rtcp_ex &= ~NSYS_RTCPEX_RTCPNEXT;
       nsys_rtcp_ex |=  NSYS_RTCPEX_RTCPSIP;
@@ -1091,8 +1080,8 @@ void nsys_netin_redirect(int fd, unsigned char * packet,
       nsys_createsip(nsys_rtcp_fd, "INFO", nsys_rtcp_info, NULL, NULL); 
     }
 
-  nsys_createsip(fd, "INVITE", invite, NULL, NULL);
-  nsys_sendsip(fd, NULL, invite);
+  nsys_createsip(fd, "INVITE", (unsigned char *) invite, NULL, NULL);
+  nsys_sendsip(fd, NULL, (unsigned char *) invite);
 
 }
 
@@ -1112,7 +1101,7 @@ void nsys_sendsip(int fd, struct sockaddr_in * addr, unsigned char * sip)
   if (addr == NULL)
     addr = (fd == nsys_rtp_fd) ? &nsys_sip_rtp_addr : &nsys_sip_rtcp_addr;
 
-  while (sendto(fd, sip, strlen(sip) + 1, 0, (struct sockaddr *) addr,
+  while (sendto(fd, sip, strlen((char *) sip) + 1, 0, (struct sockaddr *) addr,
 		sizeof(struct sockaddr)) == -1)
     {
       if (errno == EAGAIN)
