@@ -1,26 +1,42 @@
 
-
 /*
 #    Sfront, a SAOL to C translator    
 #    This file: Reads binary mp4 files
-#    Copyright (C) 1999  Regents of the University of California
 #
-#    This program is free software; you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License (Version 2) as
-#    published by the Free Software Foundation.
+# Copyright (c) 1999-2006, Regents of the University of California
+# All rights reserved.
 #
-#    This program is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#    GNU General Public License for more details.
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are
+# met:
 #
-#    You should have received a copy of the GNU General Public License
-#    along with this program; if not, write to the Free Software
-#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#  Redistributions of source code must retain the above copyright
+#  notice, this list of conditions and the following disclaimer.
+#
+#  Redistributions in binary form must reproduce the above copyright
+#  notice, this list of conditions and the following disclaimer in the
+#  documentation and/or other materials provided with the distribution.
+#
+#  Neither the name of the University of California, Berkeley nor the
+#  names of its contributors may be used to endorse or promote products
+#  derived from this software without specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #
 #    Maintainer: John Lazzaro, lazzaro@cs.berkeley.edu
 */
 
+
 #include "tree.h"
 #include "parser.tab.h"
 
@@ -404,6 +420,7 @@ void readnextsample(void)
   int hasbase, hasloop, unity, fract;
   int start = 0;
   int end = 0;
+  int floatsamples = 0;
   float base = 0.0F;
 
 
@@ -422,6 +439,10 @@ void readnextsample(void)
       u.l = readbit(32);
       base = u.f;
     }
+
+  floatsamples = readbit(1);
+  start *= (floatsamples + 2);
+  end *= (floatsamples + 2);                   
   strcat(name,".wav");
   soundfile = fopen(name,"wb");
   if (soundfile == NULL)
@@ -437,17 +458,17 @@ void readnextsample(void)
   wavefileintwrite(1,2);     /* PCM  */
   wavefileintwrite(1,2);     /* mono */
   wavefileintwrite(srate,4); /* srate */
-  wavefileintwrite(srate*2,4);  /* bytes/sec */
+  wavefileintwrite(srate*(2 + floatsamples), 4);  /* bytes/sec */
   wavefileintwrite(2,2);     /* block align 2 */
-  wavefileintwrite(16,2);     /* 2 bytes per sample */
+  wavefileintwrite(16+8*floatsamples,2);     /* 2/3 bytes per sample */
   putsoundbytes((unsigned char *) "data",4);
-  wavefileintwrite(len*2,4); /* number of bytes */
-  if (readbit(1))
+  wavefileintwrite(len*(2+floatsamples),4); /* number of bytes */
+  if (floatsamples)
     {
       while (len > 0)
 	{
 	  u.l = readbit(32);
-	  wavefileintwrite((unsigned long)(u.f*32767),2); 
+	  wavefileintwrite((long)(u.f*(pow(2,23) - 1)), 3); 
 	  len--;
 	}
     }
@@ -464,14 +485,14 @@ void readnextsample(void)
       putsoundbytes((unsigned char *) "smpl",4);
       if (hasloop)
 	wavefileintwrite(60,4); /* Size */
-      else
+      else 
 	wavefileintwrite(36,4); /* Size */
       wavefileintwrite(0,4);    /* Manufacturer */
       wavefileintwrite(0,4);    /* Product */
       wavefileintwrite(0,4);    /* Sample Period */
       if (hasbase)
 	{
-	  base =  (float)(69.0 + 1.731234e+01F*log(base*0.00227273));
+	  base =  (float)(69.0 + 1.731234e+01*log(base*0.00227273));
 	  unity = (int)base;
 	  fract = (int)((base - (float)unity)*4.295e9);
 	}
