@@ -1,26 +1,42 @@
 
-
 /*
 #    Sfront, a SAOL to C translator    
 #    This file: Writes binary mp4 files
-#    Copyright (C) 1999  Regents of the University of California
 #
-#    This program is free software; you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License (Version 2) as
-#    published by the Free Software Foundation.
+# Copyright (c) 1999-2006, Regents of the University of California
+# All rights reserved.
 #
-#    This program is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#    GNU General Public License for more details.
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are
+# met:
 #
-#    You should have received a copy of the GNU General Public License
-#    along with this program; if not, write to the Free Software
-#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#  Redistributions of source code must retain the above copyright
+#  notice, this list of conditions and the following disclaimer.
+#
+#  Redistributions in binary form must reproduce the above copyright
+#  notice, this list of conditions and the following disclaimer in the
+#  documentation and/or other materials provided with the distribution.
+#
+#  Neither the name of the University of California, Berkeley nor the
+#  names of its contributors may be used to endorse or promote products
+#  derived from this software without specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #
 #    Maintainer: John Lazzaro, lazzaro@cs.berkeley.edu
 */
 
+
 #include "tree.h"
 #include "parser.tab.h"
 
@@ -826,6 +842,7 @@ unsigned long wavefileintval(int numbytes)
 
   return ret;
 }
+
   
 /*********************************************************/
 /*     converts byte stream to an unsigned long          */
@@ -901,10 +918,17 @@ void nextsamplewrite(sigsym * sptr)
   unsigned long len = 0;
   unsigned long i, j, m, e;
   unsigned long srate = 0;  
-  int numbytes = 2;         
+  long channels = 1;         /* number of channels for the file              */
+  long numbytes = 2;         /* number of bytes in a data word               */    
+  long framebytes = 2;       /* number of bytes in one N-channel frame       */
+  long chanpoint = 0;        /* 1st byte of channel in frame; -1 sum to mono */
   sigsym * s2ptr;
   unsigned char cookie[4];
   char name[STRSIZE];
+  char * suffix;
+  char at_n = -1;
+  int ntotal = 0;
+  int extlen, padbyte, markers;
   long * sbuff = NULL;
   long unity = 0;
   long fract = 0;
@@ -925,18 +949,41 @@ void nextsamplewrite(sigsym * sptr)
 
   bitwrite(s2ptr->special, 16);  /* token */
 
-  if ((!strstr(sptr->val,".aif")) && (!strstr(sptr->val,".wav")) &&
-      (!strstr(sptr->val,".AIF")) && (!strstr(sptr->val,".WAV")) )
+  suffix = strrchr(sptr->val, '.');
+
+  if ((suffix == NULL) ||
+      !(strstr(suffix, ".wav") || strstr(suffix, ".WAV") || 
+	strstr(suffix, ".aif") || strstr(suffix, ".AIF") || 
+	strstr(suffix, ".aiff") || strstr(suffix, ".AIFF")))
     {
       printf("Error: Sample name %s lacks .wav or .aif(f) extension.\n",
 	     sptr->val);
       noerrorplace();
     }
-  if (strstr(sptr->val,".wav")||strstr(sptr->val,".WAV"))
+
+  extlen = (strstr(suffix, ".aiff") || strstr(suffix, ".AIFF")) ? 5 : 4;
+
+  if (strlen(suffix) <= extlen)          
+    soundfile = fopen(sptr->val,"rb");
+  else
     {
+      if ((sscanf(&(suffix[extlen]), "@%hhd%n", &at_n, &ntotal) < 1) || 
+	  (strlen(suffix) != extlen + ntotal) || (at_n < 0))
+	{
+	  printf("Error: Syntax error in sample filename %s (or in @N channel).\n",
+		 sptr->val);
+	  noerrorplace();
+	}
+      suffix[extlen]= '\0';
       soundfile = fopen(sptr->val,"rb");
-      if (soundfile == NULL)
-	bitsounderror(sptr);
+      suffix[extlen]= '@';
+    }
+
+  if (soundfile == NULL)
+    bitsounderror(sptr);
+
+  if (strstr(suffix,".wav")||strstr(suffix,".WAV"))
+    {
       if (soundtypecheck("RIFF"))
 	bitsounderror(sptr);
       flushsoundbytes(4);
@@ -949,28 +996,75 @@ void nextsamplewrite(sigsym * sptr)
 	bitsounderror(sptr);
       if (wavefileintval(2)!= 1) /* PCM wave files = 1 */
 	bitsounderror(sptr);
-      if (wavefileintval(2)!= 1) /* Mono files only */
+      if ((channels = wavefileintval(2)) <= 0)
 	bitsounderror(sptr);
       srate = wavefileintval(4);
       flushsoundbytes(6);
-      numbytes = (wavefileintval(2)+7)/8;   /* bytes/sample */
-      if (numbytes > 2)                     /* 1 or 2 bytes only */
-	bitsounderror(sptr);
-      flushsoundbytes(i);
+      numbytes = (wavefileintval(2)+7)/8;     /* bytes/sample */
+      if ((numbytes <= 0) || (numbytes > 3))  /* 1/2/3 bytes only */
+	{
+	  printf("Error: Sample %s isn't 8/16/24 bit.\n",
+		 sptr->val);
+	  noerrorplace();
+	}
+      framebytes = channels*numbytes;
+      if (channels == 1)
+	chanpoint = 0;
+      else
+	{
+	  if (at_n >= 0)
+	    {
+	      if (at_n > channels - 1)
+		{
+		  printf("Error: Sample generator @%i filename syntax requests\n"
+			 "       the %i'th channel of an %li channel samplefile %s.\n", 
+			 at_n, at_n+1, channels, sptr->val);
+		  bitsounderror(sptr);
+		}
+	      chanpoint = at_n*numbytes;
+	    }
+	  else
+	    chanpoint = -1;
+	}
+      flushsoundbytes(i + (i % 2));
+
       unity = fract = start = end = hasbase = hasloop = 0;
       while (!getnextcookie(cookie))
 	{
 	  if (!strncmp((char *) cookie,"data",4))
 	    {
-	      len = i = wavefileintval(4)/numbytes;
+	      padbyte = ((i = wavefileintval(4)) % 2);
+	      len = i = i/framebytes;
 	      vmcheck(sbuff = (long *) calloc(len, sizeof(long)));
 	      while (i > 0)
 		{
-		  if (numbytes == 2)
-		    sbuff[len - (i--)] = wavefileintval(numbytes);
-		  else
-		    sbuff[len - (i--)] = (wavefileintval(numbytes)-128)<<8;
+		  sbuff[len - i] = 0;
+		  for (j = 0; j < framebytes; j += numbytes)
+		    if ((j == chanpoint) || (chanpoint < 0))
+		      {
+			switch (numbytes) {
+			case 1:
+			  sbuff[len - i] += (((long)wavefileintval(1))-128)<<8;
+			  break;
+			case 2:
+			  sbuff[len - i] += ((long)wavefileintval(1));
+			  sbuff[len - i] += ((long)((char)wavefileintval(1))) << 8;
+			  break;
+			case 3:
+			  sbuff[len - i] += ((long)wavefileintval(1));
+			  sbuff[len - i] += (((long)wavefileintval(1)) << 8);
+			  sbuff[len - i] += ((long)((char)wavefileintval(1))) << 16;
+			  break;
+			}
+		      }
+		    else
+		      flushsoundbytes(numbytes);
+		  if (chanpoint < 0)
+		    sbuff[len - i] /= channels; 
+		  i--;
 		}
+	      if (padbyte)
+		flushsoundbytes(1);
 	    }
 	  else
 	    {
@@ -987,15 +1081,18 @@ void nextsamplewrite(sigsym * sptr)
 		    {
 		      hasloop = 1;
 		      flushsoundbytes(8);   /* ID, Type */ 
-		      start = wavefileintval(4);
-		      end = wavefileintval(4);
+		      start = wavefileintval(4)/framebytes;
+		      end = wavefileintval(4)/framebytes;
 		      flushsoundbytes(8);   /* Frac, count */
 		      i -= 24;
-		      flushsoundbytes(i);   /* other loops */ 
+		      flushsoundbytes(i + (i % 2));   /* other loops */ 
 		    }
 		}
 	      else
-		flushsoundbytes(wavefileintval(4));
+		{
+		  i = wavefileintval(4);
+		  flushsoundbytes(i + (i % 2));
+		}
 	    }
 	}
       bitwrite(len,24);               /* length */
@@ -1014,19 +1111,26 @@ void nextsamplewrite(sigsym * sptr)
 		    ((double)unity + ((double)fract/4.295e9) - 69.0)/12.0));
 	  bitwrite(fwr.l, 32);
 	}
-      bitwrite(0, 1);                 /* 16-bit integers */
+      if (numbytes == 3)
+	bitwrite(1, 1);              /* write 24-bit data as floats */
+      else
+	bitwrite(0, 1);              /* otherwise, as 16-bit integers */
       i = len;
       if (sbuff == NULL)
 	bitsounderror(sptr);
-      while (i > 0)
-	bitwrite(sbuff[len - (i--)], 16);
+      if (numbytes == 3)
+	while (i > 0)
+	  {
+	    fwr.f = ((float)pow(2, -23))*((float)(sbuff[len - (i--)]));
+	    bitwrite(fwr.l, 32);
+	  }
+      else
+	while (i > 0)
+	  bitwrite(sbuff[len - (i--)], 16);
       free(sbuff);
     }
   else
     {
-      soundfile = fopen(sptr->val,"rb");
-      if (soundfile == NULL)
-	bitsounderror(sptr);
       if (soundtypecheck("FORM"))
 	bitsounderror(sptr);
       flushsoundbytes(4);
@@ -1038,27 +1142,49 @@ void nextsamplewrite(sigsym * sptr)
       while (!getnextcookie(cookie))
 	if (!strncmp((char *) cookie,"COMM",4))
 	  {
-	    flushsoundbytes(4);       /* length */
-	    if (aiffileintval(2)!= 1) /* Mono files only */
+	    i = aiffileintval(4) - 18;   /* length - parsed fields */
+	    if ((channels = aiffileintval(2)) <= 0)
 	      bitsounderror(sptr);
 	    flushsoundbytes(4);       /* frames */
 	    numbytes = aiffileintval(2);
-	    if ((numbytes != 8)&&(numbytes != 16))
+	    if ((numbytes < 8)||(numbytes > 24))
 	      {
-		printf("Error: Sample %s isn't 8 or 16 bit.\n",
+		printf("Error: Sample %s isn't 8/16/24 bit.\n",
 		       sptr->val);
 		noerrorplace();
 	      }
 	    numbytes /= 8;
+	    framebytes = channels*numbytes;
 	    e = ((short)aiffileintval(2))&0x7FFF;
 	    m = aiffileintval(4);
-	    flushsoundbytes(4);
 	    srate = (unsigned long) (0.5 + (m*exp(log(2)*(e - 16414.0F))));
+	    if (channels == 1)
+	      chanpoint = 0;
+	    else
+	      {
+		if (at_n >= 0)
+		  {
+		    if (at_n > channels - 1)
+		      {
+			printf("Error: Sample generator @%i filename syntax requests\n"
+			       "       the %i'th channel of an %li channel samplefile %s.\n", 
+			       at_n, at_n+1, channels, sptr->val);
+			bitsounderror(sptr);
+		      }
+		    chanpoint = at_n*numbytes;
+		  }
+		else
+		  chanpoint = -1;
+	      }
+	    flushsoundbytes(4);
+	    if (i)
+	      flushsoundbytes(i + (i % 2));
 	    break;
 	  }
 	else
-	  {		  
-	    flushsoundbytes(aiffileintval(4));
+	  {
+	    i = aiffileintval(4);
+	    flushsoundbytes(i + (i % 2));
 	  }
       
       /* extract data from SSND, INST, MARK */
@@ -1067,23 +1193,46 @@ void nextsamplewrite(sigsym * sptr)
 	{
 	  if (!strncmp((char *) cookie,"SSND",4))
 	    {
-	      len = i = (aiffileintval(4) - 8)/numbytes;
+	      padbyte = ((i = (aiffileintval(4) - 8)) % 2);
+	      len = i = i/framebytes;
 	      flushsoundbytes(8);
 	      vmcheck(sbuff = (long *) calloc(len, sizeof(long)));
 	      while (i > 0)
 		{
-		  if (numbytes == 2)
-		    sbuff[len - (i--)] = aiffileintval(numbytes);
-		  else
-		    sbuff[len - (i--)] = aiffileintval(numbytes)<<8;
+		  sbuff[len - i] = 0;
+		  for (j = 0; j < framebytes; j += numbytes)
+		    if ((j == chanpoint) || (chanpoint < 0))
+		      {
+			switch (numbytes) {
+			case 1:
+			  sbuff[len - i] += ((long)((char)wavefileintval(1))) << 8;
+			  break;
+			case 2:
+			  sbuff[len - i] += ((long)((char)wavefileintval(1))) << 8;
+			  sbuff[len - i] += ((long)wavefileintval(1));
+			  break;
+			case 3:
+			  sbuff[len - i] += ((long)((char)wavefileintval(1))) << 16;
+			  sbuff[len - i] += (((long)wavefileintval(1)) << 8);
+			  sbuff[len - i] += ((long)wavefileintval(1));
+			  break;
+			}
+		      }
+		    else
+		      flushsoundbytes(numbytes);
+		  if (chanpoint < 0)
+		    sbuff[len - i] /= channels;
+		  i--;
 		}
+	      if (padbyte)
+		flushsoundbytes(1);
 	    }
 	  else
 	    {
 	      if (!strncmp((char *) cookie,"INST",4))
 		{
 		  hasbase = 1;
-		  flushsoundbytes(4); /* size */
+		  i = aiffileintval(4) - 20;  /* length - parsed fields */
 		  unity = aiffileintval(1); /* baseNote */
 		  getsoundsbytes(&c,1);     /* sbytes .. because signed */
 		  fract = c;                /* detune */
@@ -1096,17 +1245,21 @@ void nextsamplewrite(sigsym * sptr)
 		  start = aiffileintval(2); /* ID Marker for start */
 		  end = aiffileintval(2);   /* ID Marker for end   */
 		  flushsoundbytes(6);       /* release loop        */
+
+		  if (i)
+		    flushsoundbytes(i + (i % 2));
 		}
 	      else
 		{
 		  if (!strncmp((char *) cookie,"MARK",4))
 		    {
-		      flushsoundbytes(4); /* size */
-		      i = aiffileintval(2); /* number of markers */
+		      i = aiffileintval(4);   /* size */
+		      markers = aiffileintval(2); /* number of markers */
+		      i -= 2;
 		      
 		      /* inserts marker positions into mhead list */
-		      
-		      while (i--)
+
+		      while (markers--)
 			{
 			  vmcheck(mptr = (aiffmark *)malloc(sizeof(aiffmark)));
 			  mptr->id = (short) aiffileintval(2);
@@ -1117,10 +1270,16 @@ void nextsamplewrite(sigsym * sptr)
 			  if (!(j&0x0001)) /* pad byte if even */
 			    j++;
 			  flushsoundbytes(j);
+			  i -= 7 + j;
 			}
+		      if (i)
+			flushsoundbytes(i + (i % 2));
 		    }
 		  else
-		    flushsoundbytes(aiffileintval(4));
+		    {
+		      i = aiffileintval(4);
+		      flushsoundbytes(i + (i % 2));
+		    }
 		}
 	    }
 	}
@@ -1178,13 +1337,24 @@ void nextsamplewrite(sigsym * sptr)
 		    ((double)unity + ((double)fract/100.0) - 69.0)/12.0));
 	  bitwrite(fwr.l, 32);
 	}
-      bitwrite(0, 1);                 /* 16-bit integers */
-      
+
+
+      if (numbytes == 3)
+	bitwrite(1, 1);              /* write 24-bit data as floats */
+      else
+	bitwrite(0, 1);              /* otherwise, as 16-bit integers */
       i = len;
       if (sbuff == NULL)
 	bitsounderror(sptr);
-      while (i > 0)
-	bitwrite(sbuff[len - (i--)], 16);    
+      if (numbytes == 3)
+	while (i > 0)
+	  {
+	    fwr.f = ((float)pow(2, -23))*((float)(sbuff[len - (i--)]));
+	    bitwrite(fwr.l, 32);
+	  }
+      else
+	while (i > 0)
+	  bitwrite(sbuff[len - (i--)], 16);
       free(sbuff);
 
       mptr = mhead;
