@@ -1,22 +1,37 @@
 
-
 /*
 #    Sfront, a SAOL to C translator    
 #    This file: Contains main()
-#    Copyright (C) 1999  Regents of the University of California
 #
-#    This program is free software; you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License (Version 2) as
-#    published by the Free Software Foundation.
+# Copyright (c) 1999-2006, Regents of the University of California
+# All rights reserved.
 #
-#    This program is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#    GNU General Public License for more details.
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are
+# met:
 #
-#    You should have received a copy of the GNU General Public License
-#    along with this program; if not, write to the Free Software
-#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#  Redistributions of source code must retain the above copyright
+#  notice, this list of conditions and the following disclaimer.
+#
+#  Redistributions in binary form must reproduce the above copyright
+#  notice, this list of conditions and the following disclaimer in the
+#  documentation and/or other materials provided with the distribution.
+#
+#  Neither the name of the University of California, Berkeley nor the
+#  names of its contributors may be used to endorse or promote products
+#  derived from this software without specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #
 #    Maintainer: John Lazzaro, lazzaro@cs.berkeley.edu
 */
@@ -229,32 +244,42 @@ void sfrontshutdown(void)
 
 
 /****************************************************************/
-/*             print gpl license brief                          */
+/*                print bsd license                           */
 /****************************************************************/
 
-void printgpl(void)
+void printbsd(void)
 
 {
 
-  printf("Copyright 1999-2001 Regents of the University of California.\n");
+  printf("Copyright (c) 1999-2006, Regents of the University of California\n");
+  printf("All rights reserved.\n");
   printf("\n");
-  printf("This program is free software; you can redistribute it\n");
-  printf("and/or modify it under the terms of the GNU General\n");
-  printf("Public License (Version 2) as published by the Free\n");
-  printf("Software Foundation.\n");
+  printf("Redistribution and use in source and binary forms, with or without\n");
+  printf("modification, are permitted provided that the following conditions are\n");
+  printf("met:\n");
   printf("\n");
-  printf("This program is distributed in the hope that it will\n");
-  printf("be useful, but WITHOUT ANY WARRANTY; without even the\n");
-  printf("implied warranty of MERCHANTABILITY or FITNESS FOR A\n");
-  printf("PARTICULAR PURPOSE.  See the GNU General Public License\n");
-  printf("for more details.\n");
+  printf(" Redistributions of source code must retain the above copyright\n");
+  printf(" notice, this list of conditions and the following disclaimer.\n");
   printf("\n");
-  printf("You should have received a copy of the GNU General\n");
-  printf("Public License along with this program; if not, write\n");
-  printf("to the Free Software Foundation, Inc., 59 Temple Place,\n");
-  printf("Suite 330, Boston, MA  02111-1307  USA In the sfront\n");
-  printf("distribution, see file COPYING for the GNU General\n");
-  printf("Public License.\n");
+  printf(" Redistributions in binary form must reproduce the above copyright\n");
+  printf(" notice, this list of conditions and the following disclaimer in the\n");
+  printf(" documentation and/or other materials provided with the distribution.\n");
+  printf("\n");
+  printf(" Neither the name of the University of California, Berkeley nor the\n");
+  printf(" names of its contributors may be used to endorse or promote products\n");
+  printf(" derived from this software without specific prior written permission.\n");
+  printf("\n");
+  printf("THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS\n");
+  printf("\"AS IS\" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT\n");
+  printf("LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR\n");
+  printf("A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT\n");
+  printf("OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,\n");
+  printf("SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT\n");
+  printf("LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,\n");
+  printf("DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY\n");
+  printf("THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT\n");
+  printf("(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE\n");
+  printf("OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.\n");
 
 }
 
@@ -312,6 +337,8 @@ void printhelp(void)
   printf("       [-render]           Accurately compute audio output.\n");
   printf("       [-playback]         Real-time streaming audio output.\n");
   printf("       [-timesync]         Real-time interactive option.\n");
+  printf("Audio output latency (caution: default value is usually best):\n");
+  printf("       [-latency time]     Soundcard latency, in seconds.\n");
   printf("To create an MP4 file, use:\n");
   printf("       [-bitout mp4file]   Generate mp4 bitstream\n");
   printf("       [-symtab]           Include symbol table in file\n");
@@ -351,7 +378,10 @@ void printhelp(void)
       printf("       [-m_semitones num]  Mirror session pitch shift (default %hu).\n",
 	     MSESSION_INTERVAL);
     }
-
+  printf("Sample-rate interpolation options:\n");
+  printf("       [-interp [linear, sinc]]  Override SAOL global interp parameter.\n");
+  printf("       [-sinc_pilen n]     Lookup table size for 1 cycle of sinc sinusoid.\n");
+  printf("       [-sinc_zcross n]    Number of zero-crossings on sinc-function lobe.\n");
   printf("Miscellanous options:\n");
   printf("       [-null-program]     Map unassigned MIDI presets to silence.\n");
   printf("       [-mv]               Verbose -midi or -mstr file read.\n");
@@ -731,7 +761,8 @@ int cmddriveropen(int od, char * command, char * name, int check)
 int oneparamflags(char * s1, char * s2)
 
 {  
-  int od, i, dots; 
+  int od, i, q1, q2, q3, q4; 
+  float f;
   char * name1, * name2;
 
   /* all files */
@@ -836,6 +867,29 @@ int oneparamflags(char * s1, char * s2)
       return 1;
     }
 
+  if ((!strcmp(s1,"-interp")))
+    {
+      if (!s2)
+	{
+	  printf("Error: Missing -interp type [linear | sinc].\n\n");
+	  printhelpshort();
+	}
+
+      if (!strcmp("linear", s2))
+	interp_cmdline = INTERP_LINEAR;
+      else
+	if (!strcmp("sinc", s2))
+	  interp_cmdline = INTERP_SINC;
+	else
+	  {
+	    printf("Error: Invalid -interp level %s; use one of\n"
+		   "[linear, sinc]\n\n", s2);
+	    printhelpshort();
+	  } 
+      
+      return 1;
+    }
+
   if (!strcmp(s1,"-latency"))
     {
        
@@ -922,58 +976,68 @@ int oneparamflags(char * s1, char * s2)
       return 1;
     }
 
+  if (!strcmp(s1,"-sinc_zcross"))
+    {
+      if (!s2 || (sscanf(s2, "%f", &f) != 1))
+	{
+	  printf("Error: Missing or illegal -sinc_zcross parameter.\n\n");
+	  printhelpshort();
+	}
+      sinc_zcross = (unsigned int) f;
+      if ((f != sinc_zcross) || (sinc_zcross < 1) || (sinc_zcross > 30))
+	{
+	  printf("Error: -sinc_zcross parameter must be integer in 1-30 range.\n\n");
+	  printhelpshort();
+	}
+      return 1;
+    }
+
+  if (!strcmp(s1,"-sinc_pilen"))
+    {
+      if (!s2 || (sscanf(s2, "%f", &f) != 1))
+	{
+	  printf("Error: Missing or illegal -sinc_pilen parameter.\n\n");
+	  printhelpshort();
+	}
+      sinc_pilen = (unsigned int) f;
+      if ((f != sinc_pilen) || 
+	  ((sinc_pilen != 2) && (sinc_pilen != 4) && (sinc_pilen != 8) && 
+	   (sinc_pilen != 16) && (sinc_pilen != 32) && (sinc_pilen != 64) &&
+	   (sinc_pilen != 128) && (sinc_pilen != 256) && (sinc_pilen != 512) &&
+	   (sinc_pilen != 1024) && (sinc_pilen != 2048) && (sinc_pilen != 4096) &&
+	   (sinc_pilen != 8192)))
+	{
+	  printf("Error: -sinc_pilen parameter must be <= 8192 and a power of 2.\n\n");
+	  printhelpshort();
+	}
+      return 1;
+    }
+
   if ((NET_STATUS == HAS_NETWORKING) && (!strcmp(s1,"-sip_ip")))
     {
-      if ((!s2) || (strlen(s2) > 15) || (strlen(s2) < 7))
+      if (!s2)
 	{
 	  printf("Error: Missing or illegal -sip_ip parameter.\n\n");
 	  printhelpshort();
 	}
 
-      od = dots = 0;
-      for (i=0; i < ((int)strlen(s2)); i++)
-	if (isdigit((int)s2[i]) || (s2[i] == '.'))
-	  {
-	    if (s2[i] == '.')
-	      {
-		if ((od < 1) || (od > 3) || (dots > 2))
-		  {
-		    printf("Error: Bad format for -sip_ip" 
-			   " dotted-quad: %s.\n\n", s2);
-		    printhelpshort();
-		  }
-		else
-		  {
-		    dots++;
-		    od = 0;
-		  }
-	      }
-	    else
-	      {
-		od++;
-		if ((od > 3) || ((od == 2) && (s2[i-1] == '0')) ||
-		    ((od == 3) && 
-		     ((s2[i-2] > '2') || 
-		      ((s2[i-2] == '2') && ((s2[i] > '5') || (s2[i-1] > '5'))))
-		     ))
-		  {
-		    printf("Error: Bad format for -sip_ip" 
-			   " dotted-quad: %s.\n\n", s2);
-		    printhelpshort();
-		  }
-	      }
-	  }
-	else
-	  {
-	    printf("Error: Bad format for -sip_ip dotted-quad: %s.\n\n", s2);
-	    printhelpshort();
-	  }
+      q1 = q2 = q3 = q4 = -1;
 
-      if ((od < 1) || (dots != 3))
+      if (sscanf(s2, "%3i.%3i.%3i.%3i", &q1, &q2, &q3, &q4) != 4)
 	{
-	  printf("Error: Bad format for -sip_ip dotted-quad: %s.\n\n", s2);
+	  printf("Error: Bad format (1) for -sip_ip" 
+		 " dotted-quad: %s.\n\n", s2);
 	  printhelpshort();
 	}
+
+      if ((q1 < 0) || (q1 > 255) || (q2 < 0) || (q2 > 255) ||
+	  (q3 < 0) || (q3 > 255) || (q4 < 0) || (q4 > 255))
+	{
+	  printf("Error: Bad format (2) for -sip_ip" 
+		 " dotted-quad: %s.\n\n", s2);
+	  printhelpshort();
+	}
+
       strcpy(sip_ip, s2);
       return 1;
     }
@@ -990,7 +1054,6 @@ int oneparamflags(char * s1, char * s2)
       return 1;
     }
 
-
   if (!strcmp(s1,"-Is"))
     {
       if (!s2)
@@ -1119,7 +1182,7 @@ int singleflags(char * s)
   if ((!strcmp(s,"-lics"))||(!strcmp(s,"-lisc"))||
       (!strcmp(s,"-license")))
     {
-      printgpl();
+      printbsd();
       noerrorplace();
     }
   if (!strcmp(s,"-mv"))
