@@ -1,28 +1,46 @@
+
 /*
-#    Sfront, a SAOL to C translator
+#    Sfront, a SAOL to C translator    
 #    This file: Include file for shared variables
-#    Copyright (C) 1999  Regents of the University of California
 #
-#    This program is free software; you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License (Version 2) as
-#    published by the Free Software Foundation.
+# Copyright (c) 1999-2006, Regents of the University of California
+# All rights reserved.
 #
-#    This program is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#    GNU General Public License for more details.
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are
+# met:
 #
-#    You should have received a copy of the GNU General Public License
-#    along with this program; if not, write to the Free Software
-#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#  Redistributions of source code must retain the above copyright
+#  notice, this list of conditions and the following disclaimer.
+#
+#  Redistributions in binary form must reproduce the above copyright
+#  notice, this list of conditions and the following disclaimer in the
+#  documentation and/or other materials provided with the distribution.
+#
+#  Neither the name of the University of California, Berkeley nor the
+#  names of its contributors may be used to endorse or promote products
+#  derived from this software without specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #
 #    Maintainer: John Lazzaro, lazzaro@cs.berkeley.edu
 */
 
+
 #ifndef _SFRONT_TREE_H
 #define _SFRONT_TREE_H 1
 
-#define IDSTRING "0.87 10/15/04"
+#define IDSTRING "0.91 07/30/06"
 
 #include <stdio.h>
 #include <math.h>
@@ -470,6 +488,13 @@ extern void * alloca(size_t);
 
 #define FFTTABSIZE 14
 
+/* sample-rate interpolation */
+
+#define INTERP_LINEAR  0
+#define INTERP_SINC    1
+
+#define PI   (3.14159265358979323846)
+
 /* print buffer size */
 
 #define ZSIZE        8196
@@ -651,6 +676,12 @@ extern void * alloca(size_t);
 #define PASSIVE_FLOW 0
 #define  ACTIVE_FLOW 1
 
+/* output file wordsize hint for drivers */
+
+#define WORDSIZE_8BIT    0
+#define WORDSIZE_16BIT   1
+#define WORDSIZE_24BIT   2
+
 /* bit positions for instr status word */
 
 #define STATWORD_EFFECTS    1
@@ -1061,6 +1092,8 @@ extern int ainlatency;
 extern int ainflow;
 extern char * ainname;
 
+extern int outfile_wordsize;
+
 extern int cin;
 extern int cmidi;
 extern int csasl;
@@ -1120,6 +1153,11 @@ extern int outchannels;
 extern int interp;
 extern float globaltune;
 
+extern int interp_cmdline;
+extern unsigned long sinc_pilen;
+extern unsigned long sinc_zcross;
+extern float sinc_upmax;
+
 extern tnode * troot;
 extern tnode * groot;
 extern tnode * instances;
@@ -1322,7 +1360,7 @@ extern char * saolsourcefile;
 /* maximum size of context-dependent lexical look-ahead */
 
 #define LEXSTACKSIZE 256
-#define LEXBUFSIZE   128
+#define LEXBUFSIZE   256
 
 extern int lexstackret[];
 extern tnode * lexstacktnode[];
@@ -1359,8 +1397,10 @@ extern int sendsemicoloncount;
 
 typedef struct sampleinfo {
 
-  char wav;            /* 1 if from wav file, 0 if from aif file */
-  char numbytes;       /* number of bytes in a data word         */
+  char wav;             /* 1 if from wav file, 0 if from aif file */
+  char numbytes;        /* number of bytes in a data word         */
+  char framebytes;      /* number of bytes in one N-channel frame */
+  char chanpoint;       /* 1st byte of channel in frame; -1 sum to mono */
   unsigned long len;    /* number of samples                      */
   unsigned long point;  /* number of bytes before first data byte */
   unsigned long srate;  /* sampling rate */
@@ -1452,6 +1492,7 @@ extern void coreopcodedoinline(tnode *);
 
 extern void coreopcodevars(tnode *);
 extern int coreopcodename(tnode *);
+extern int coreopcodehasextras(tnode *);
 extern int coreopcodespeedtrap(tnode *);
 extern int delicatepolyops(tnode * tptr);
 extern int polyopcallexcept(tnode * tptr);
@@ -1697,7 +1738,7 @@ extern void haswavegenerator(tnode *);
 extern float * wavereduceconstants(tnode *, tnode *);
 extern void wavegeneratorvar(sigsym *);
 extern void createtable(sigsym *, char *, int);
-
+extern void tablestartupcheck(sigsym * ident, int mode, int kind);
 /* from wtconst.c */
 
 extern float * wavereduceconstants(tnode *, tnode *);
