@@ -1,26 +1,42 @@
 
-
 /*
 #    Sfront, a SAOL to C translator    
 #    This file: Code generation: user-defined opcodes
-#    Copyright (C) 1999  Regents of the University of California
 #
-#    This program is free software; you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License (Version 2) as
-#    published by the Free Software Foundation.
+# Copyright (c) 1999-2006, Regents of the University of California
+# All rights reserved.
 #
-#    This program is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#    GNU General Public License for more details.
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are
+# met:
 #
-#    You should have received a copy of the GNU General Public License
-#    along with this program; if not, write to the Free Software
-#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#  Redistributions of source code must retain the above copyright
+#  notice, this list of conditions and the following disclaimer.
+#
+#  Redistributions in binary form must reproduce the above copyright
+#  notice, this list of conditions and the following disclaimer in the
+#  documentation and/or other materials provided with the distribution.
+#
+#  Neither the name of the University of California, Berkeley nor the
+#  names of its contributors may be used to endorse or promote products
+#  derived from this software without specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #
 #    Maintainer: John Lazzaro, lazzaro@cs.berkeley.edu
 */
 
+
 #include "tree.h"
 #include "parser.tab.h"
 
@@ -183,12 +199,16 @@ void redefglobal(void)
   fprintf(outfile,"#define NV(x)  global[x].f\n");
   fprintf(outfile,"#undef NVI\n");
   fprintf(outfile,"#define NVI(x)  global[x].i\n");
+  fprintf(outfile,"#undef NVUI\n");
+  fprintf(outfile,"#define NVUI(x)  global[x].ui\n");
   fprintf(outfile,"#undef NVU\n");
   fprintf(outfile,"#define NVU(x)  global[x]\n");
   fprintf(outfile,"#undef NP\n");
   fprintf(outfile,"#define NP(x)  global[x].f\n");
   fprintf(outfile,"#undef NPI\n");
   fprintf(outfile,"#define NPI(x)  global[x].i\n");
+  fprintf(outfile,"#undef NPUI\n");
+  fprintf(outfile,"#define NPUI(x)  global[x].ui\n");
   fprintf(outfile,"\n");
 }
 
@@ -211,12 +231,16 @@ void redefnormal(void)
   fprintf(outfile,"#define NV(x)  nstate->v[x].f\n");
   fprintf(outfile,"#undef NVI\n");
   fprintf(outfile,"#define NVI(x)  nstate->v[x].i\n");
+  fprintf(outfile,"#undef NVUI\n");
+  fprintf(outfile,"#define NVUI(x)  nstate->v[x].ui\n");
   fprintf(outfile,"#undef NVU\n");
   fprintf(outfile,"#define NVU(x)  nstate->v[x]\n");
   fprintf(outfile,"#undef NP\n");
   fprintf(outfile,"#define NP(x)  nstate->v[x].f\n");
   fprintf(outfile,"#undef NPI\n");
   fprintf(outfile,"#define NPI(x)  nstate->v[x].i\n");
+  fprintf(outfile,"#undef NPUI\n");
+  fprintf(outfile,"#define NPUI(x)  nstate->v[x].ui\n");
   fprintf(outfile,"\n");
 }
 
@@ -239,12 +263,16 @@ void redefstatic(int i)
   fprintf(outfile,"#define NV(x)  ninstr[%i].v[x].f\n",i);
   fprintf(outfile,"#undef NVI\n");
   fprintf(outfile,"#define NVI(x)  ninstr[%i].v[x].i\n",i);
+  fprintf(outfile,"#undef NVUI\n");
+  fprintf(outfile,"#define NVUI(x)  ninstr[%i].v[x].ui\n",i);
   fprintf(outfile,"#undef NVU\n");
   fprintf(outfile,"#define NVU(x)  ninstr[%i].v[x]\n",i);
   fprintf(outfile,"#undef NP\n");
   fprintf(outfile,"#define NP(x)  ninstr[%i].v[x].f\n",i);
   fprintf(outfile,"#undef NPI\n");
   fprintf(outfile,"#define NPI(x)  ninstr[%i].v[x].i\n",i);
+  fprintf(outfile,"#undef NPUI\n");
+  fprintf(outfile,"#define NPUI(x)  ninstr[%i].v[x].ui\n",i);
   fprintf(outfile,"\n");
 }
 
@@ -680,7 +708,7 @@ void printopargs(tnode * tcall, tnode * aptr, tnode * fptr,
 		{
 		  /* tablemaps */
 		  
-		  sprintf(tablename,"%s",fptr->down->next->down->val);
+		  sprintf(tablename,"va_%s",fptr->down->next->down->val);
 		  printtmapcase(aptr->down, aptr->down->next->next, tablename);
 		  printtabledefine(tablenum++, tablename, S_TABLEMAP, -1);
 		}
@@ -748,6 +776,10 @@ void printopimports(tnode * tptr, char * newfix)
 	      createtable(sptr,name, S_OPCODE);
 	      fprintf(outfile, "   }\n");
 	    }
+
+	  if ((sptr->kind == K_IMPORTEXPORT) && startupinstr && 
+	      (currinstrument == startupinstr))
+	    tablestartupcheck(sptr, S_OPCODE, K_IMPORTEXPORT); 
 	}
       else
 	{
@@ -1218,6 +1250,7 @@ void redefoparrayargs(void)
   fprintf(outfile,"\n");
   fprintf(outfile,"#undef NP\n");
   fprintf(outfile,"#undef NPI\n");
+  fprintf(outfile,"#undef NPUI\n");
 
   fprintf(outfile,"#define NP(x) %s \n", 
 	  (pstr = stackstring(S_NUMBER, curroparraydepth, "x")));
@@ -1227,6 +1260,10 @@ void redefoparrayargs(void)
 	  (pstr = stackstring(S_INTGR, curroparraydepth, "x")));
   free(pstr);
 
+  fprintf(outfile,"#define NPUI(x) %s \n\n", 
+	  (pstr = stackstring(S_FREE1, curroparraydepth, "x")));
+  free(pstr);
+
 }
 
 
@@ -1243,6 +1280,7 @@ void redefoparraybody(void)
   fprintf(outfile,"#undef NT\n");
   fprintf(outfile,"#undef NV\n");
   fprintf(outfile,"#undef NVI\n");
+  fprintf(outfile,"#undef NVUI\n");
   fprintf(outfile,"#undef NVU\n");
 
   fprintf(outfile,"#define NV(x) %s \n", 
@@ -1253,6 +1291,10 @@ void redefoparraybody(void)
 	  (pstr = stackstring(S_INTGR, curroparraydepth, "x")));
   free(pstr);
 
+  fprintf(outfile,"#define NVUI(x) %s \n", 
+	  (pstr = stackstring(S_FREE1, curroparraydepth, "x")));
+  free(pstr);
+
   fprintf(outfile,"#define NVU(x) %s \n", 
 	  (pstr = stackstring(S_UNION, curroparraydepth, "x")));
   free(pstr);
@@ -1868,6 +1910,9 @@ char * stackstring(int type, int depth, char * idxstr)
 	  case S_INTGR:
 	    sprintf(retstr, "ninstr[%i].v[%s].i", i, idxstr);
 	    break;
+	  case S_FREE1:
+	    sprintf(retstr, "ninstr[%i].v[%s].ui", i, idxstr);
+	    break;
 	  case S_UNION:
 	    sprintf(retstr, "ninstr[%i].v[%s]", i, idxstr);
 	    break;
@@ -1887,6 +1932,9 @@ char * stackstring(int type, int depth, char * idxstr)
 	  case S_INTGR:
 	    sprintf(retstr, "(ninstr[%i].vstate)[%i][%s].i", i, depth, idxstr);
 	    break;
+	  case S_FREE1:
+	    sprintf(retstr, "(ninstr[%i].vstate)[%i][%s].ui", i, depth, idxstr);
+	    break;
 	  case S_UNION:
 	    sprintf(retstr, "(ninstr[%i].vstate)[%i][%s]", i, depth, idxstr);
 	    break;
@@ -1911,6 +1959,9 @@ char * stackstring(int type, int depth, char * idxstr)
 	      case S_INTGR:
 		sprintf(retstr, "nstate->v[%s].i", idxstr);
 		break;
+	      case S_FREE1:
+		sprintf(retstr, "nstate->v[%s].ui", idxstr);
+		break;
 	      case S_UNION:
 		sprintf(retstr, "nstate->v[%s]", idxstr);
 		break;
@@ -1930,6 +1981,9 @@ char * stackstring(int type, int depth, char * idxstr)
 	      case S_INTGR:
 		sprintf(retstr, "(nstate->vstate)[%i][%s].i", depth, idxstr);
 		break;
+	      case S_FREE1:
+		sprintf(retstr, "(nstate->vstate)[%i][%s].ui", depth, idxstr);
+		break;
 	      case S_UNION:
 		sprintf(retstr, "(nstate->vstate)[%i][%s]", depth, idxstr);
 		break;
@@ -1952,6 +2006,9 @@ char * stackstring(int type, int depth, char * idxstr)
 	      case S_INTGR:
 		sprintf(retstr, "global[%s].i", idxstr);
 		break;
+	      case S_FREE1:
+		sprintf(retstr, "global[%s].ui", idxstr);
+		break;
 	      case S_UNION:
 		sprintf(retstr, "global[%s]", idxstr);
 		break;
@@ -1971,6 +2028,9 @@ char * stackstring(int type, int depth, char * idxstr)
 	      case S_INTGR:
 		sprintf(retstr, "globalvstate[%i][%s].i", depth, idxstr);
 		break;
+	      case S_FREE1:
+		sprintf(retstr, "globalvstate[%i][%s].ui", depth, idxstr);
+		break;
 	      case S_UNION:
 		sprintf(retstr, "globalvstate[%i][%s]", depth, idxstr);
 		break;
