@@ -1,26 +1,42 @@
 
-
 /*
 #    Sfront, a SAOL to C translator    
 #    This file: Code generaton: core wavetables
-#    Copyright (C) 1999  Regents of the University of California
 #
-#    This program is free software; you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License (Version 2) as
-#    published by the Free Software Foundation.
+# Copyright (c) 1999-2006, Regents of the University of California
+# All rights reserved.
 #
-#    This program is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#    GNU General Public License for more details.
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are
+# met:
 #
-#    You should have received a copy of the GNU General Public License
-#    along with this program; if not, write to the Free Software
-#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#  Redistributions of source code must retain the above copyright
+#  notice, this list of conditions and the following disclaimer.
+#
+#  Redistributions in binary form must reproduce the above copyright
+#  notice, this list of conditions and the following disclaimer in the
+#  documentation and/or other materials provided with the distribution.
+#
+#  Neither the name of the University of California, Berkeley nor the
+#  names of its contributors may be used to endorse or promote products
+#  derived from this software without specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #
 #    Maintainer: John Lazzaro, lazzaro@cs.berkeley.edu
 */
 
+
 #include "tree.h"
 
 
@@ -284,11 +300,25 @@ void createconstanttable(sigsym * ident, char * prefix, int mode)
   mz(lc); sprintf(z[lc++], 
 		  "NT(%s_%s).lenf = (float)(NT(%s_%s).len = %i);",
 		   prefix,ident->val, prefix, ident->val, size);
+
   mz(lc); sprintf(z[lc++], "NT(%s_%s).tend = %i;",
 		  prefix,ident->val, size -1);
-  mz(lc); sprintf(z[lc++], "NT(%s_%s).n = NT(%s_%s).diff = %f;",
-		  prefix,ident->val, prefix,ident->val, 
-		  (size - 1.0F)/size);
+
+  mz(lc); sprintf(z[lc++], 
+		  "NT(%s_%s).oconst = %i.0F*ATIME;",
+		   prefix,ident->val, size);
+
+  if (interp == INTERP_SINC)
+    {
+      mz(lc); sprintf(z[lc++], 
+		      "NT(%s_%s).sffl = 1.0F;", prefix,ident->val);
+      
+      mz(lc); sprintf(z[lc++], 
+		      "NT(%s_%s).sfui = 0x00010000;", prefix,ident->val);
+      
+      mz(lc); sprintf(z[lc++], 
+		      "NT(%s_%s).dsincr = SINC_PILEN;", prefix,ident->val);
+    }
 
   switch (mode) {
   case S_GLOBAL:
@@ -1382,22 +1412,18 @@ void sampleconstcheck(tnode * tptr, tnode * nsl)
 
   suffix = strrchr(tptr->val, '.');
 
-  if ((suffix == NULL) || (strlen(suffix) < 4) || (strlen(suffix) > 5)) 
+  if ((suffix == NULL) ||
+      !(strstr(suffix, ".wav") || strstr(suffix, ".WAV") || 
+	strstr(suffix, ".aif") || strstr(suffix, ".AIF") || 
+	strstr(suffix, ".aiff") || strstr(suffix, ".AIFF")))
     {
-      printf("Error: Sample generator filename requires .wav/.aif suffix.\n");
+      printf("Error: Sample generator requires "
+	     ".wav/.WAV/.aif/.AIF/.aiff/.AIFF file name extension.\n");
       printtableproblem(tptr, nsl);
     }
 
   wav = (strstr(suffix, ".wav") != NULL) || (strstr(suffix, ".WAV") != NULL);
 
-  if ((!wav) && 
-      (strstr(suffix, ".aif") == NULL) && (strstr(suffix, ".AIF") == NULL) &&
-      (strstr(suffix, ".aiff") == NULL) && (strstr(suffix, ".AIFF") == NULL))
-    {
-      printf("Error: Sample generator filename requires .wav/.aif suffix.\n");
-      printtableproblem(tptr, nsl);
-    }
-
   /*************/
   /* read file */
   /*************/
@@ -1926,13 +1952,32 @@ void wavefileconstcheck(tnode * tptr, tnode * nsl)
   long unity = 0;
   long fract = 0;
   int dataflag = 0;
+  long channels = 0;
+  char * suffix;
+  char at_n = -1;
+  int ntotal = 0;
 
   vmcheck(sinfo = (sampleinfo *) calloc(1, sizeof(sampleinfo)));
   tptr->ibus = (tnode *) sinfo;
 
   sinfo->wav = 1;
 
-  soundfile = fopen(tptr->val,"rb");
+  suffix = strrchr(tptr->val, '.');  /* prechecked to exist */
+
+  if (strlen(suffix) <= 4)          /* if true, prechecked == 4 */
+    soundfile = fopen(tptr->val,"rb");
+  else
+    {
+      if ((sscanf(&(suffix[4]), "@%hhd%n", &at_n, &ntotal) < 1) || 
+	  (strlen(suffix) != 4 + ntotal) || (at_n < 0))
+	{
+	  printf("Error: Syntax error in sample filename (or in @N specifier).\n");
+	  printtableproblem(tptr, nsl);
+	}
+      suffix[4]= '\0';
+      soundfile = fopen(tptr->val,"rb");
+      suffix[4]= '@';
+    }
 
   /*******************/
   /* read WAV header */
@@ -1958,20 +2003,38 @@ void wavefileconstcheck(tnode * tptr, tnode * nsl)
       printf("Error: Sfront only handles PCM-format WAV files\n");
       printtableproblem(tptr, nsl);
     }
-  if (wt_wavefileintval(2, &count, tptr, nsl)!= 1) /* Mono files only */
-    {
-      printf("Error: Sample generator can't read stereo WAV files\n");
-      printtableproblem(tptr, nsl);
-    }
+  channels = wt_wavefileintval(2, &count, tptr, nsl);  /* 1 = mono, 2 = stereo */
   sinfo->srate = wt_wavefileintval(4, &count, tptr, nsl);
   wt_flushsoundbytes(6, &count, tptr, nsl);
   sinfo->numbytes = (char)((wt_wavefileintval(2, &count, tptr, nsl)+7)/8);  
-  if (sinfo->numbytes > 2)  /* 1 or 2 bytes-per-sample only */
+  if ((sinfo->numbytes <= 0) || (sinfo->numbytes > 3)) 
     {
-      printf("Error: Sample generator only handles 8 and 16 bit WAV files.\n");
+      printf("Error: Sample generator only handles 8/16/24-bit WAV files.\n");
       printtableproblem(tptr, nsl);
     }
-  wt_flushsoundbytes(i, &count, tptr, nsl);
+  sinfo->framebytes = channels * sinfo->numbytes;
+
+  if (channels == 1)
+    sinfo->chanpoint = 0;
+  else
+    {
+      if (at_n >= 0)
+	{
+	  if (at_n > channels - 1)
+	    {
+	      printf("Error: Sample generator @%i filename syntax requests\n"
+		     "       the %i'th channel of an %li channel WAV file.\n", 
+		     at_n, at_n+1, channels);
+	      printtableproblem(tptr, nsl);
+	    }
+	  sinfo->chanpoint = at_n*sinfo->numbytes;
+	}
+      else
+	sinfo->chanpoint = -1;
+    }
+
+  if (i)
+    wt_flushsoundbytes(i + (i % 2), &count, tptr, nsl);
 
 
   /************************************************/
@@ -1988,10 +2051,10 @@ void wavefileconstcheck(tnode * tptr, tnode * nsl)
 	  /* save seek point, flush data */
 	  /*******************************/
 
-	  sinfo->len = 
-	    wt_wavefileintval(4, &count, tptr, nsl)/(sinfo->numbytes);
+	  sinfo->len = wt_wavefileintval(4, &count, tptr, nsl);
 	  sinfo->point = count;   /* where sa.c file should seek to */
-	  wt_flushsoundbytes(sinfo->numbytes*sinfo->len, &count, tptr, nsl);
+	  wt_flushsoundbytes(sinfo->len + (sinfo->len % 2), &count, tptr, nsl);
+	  sinfo->len /= sinfo->framebytes;
 	  dataflag = 1;
 	}
       else
@@ -2031,14 +2094,18 @@ void wavefileconstcheck(tnode * tptr, tnode * nsl)
 		  wt_flushsoundbytes(8, &count, tptr, nsl); /* ID, Type */ 
 		  sinfo->start = wt_wavefileintval(4, &count, tptr, nsl);
 		  sinfo->end = wt_wavefileintval(4, &count, tptr, nsl);
+		  sinfo->start /= sinfo->framebytes;
+		  sinfo->end /= sinfo->framebytes;
 		  wt_flushsoundbytes(8, &count, tptr, nsl); /* Frac, count */
 		  i -= 24;
-		  wt_flushsoundbytes(i, &count, tptr, nsl); /* other loops */ 
+		  wt_flushsoundbytes(i + (i % 2), &count, tptr, nsl); /* other loops */ 
 		}
 	    }
 	  else
-	    wt_flushsoundbytes(wt_wavefileintval(4, &count, tptr, nsl),
-			       &count, tptr, nsl);
+	    {
+	      i = wt_wavefileintval(4, &count, tptr, nsl);
+	      wt_flushsoundbytes(i + (i % 2), &count, tptr, nsl);
+	    }
 	}
     }
 
@@ -2060,16 +2127,42 @@ void aiffileconstcheck(tnode * tptr, tnode * nsl)
   unsigned char cookie[4];  
   int dataflag = 0;
   unsigned long count = 0;
+  unsigned long chunksize;
   long unity = 0;
   long fract = 0;
   char c;
   struct aiffmark * mptr = NULL;
   struct aiffmark * mhead = NULL;
+  long channels = 0;
+  char * suffix;
+  char at_n = -1;
+  int ntotal = 0;
+  int extlen;
 
   vmcheck(sinfo = (sampleinfo *) calloc(1, sizeof(sampleinfo)));
   tptr->ibus = (tnode *) sinfo;
 
-  soundfile = fopen(tptr->val,"rb");
+  suffix = strrchr(tptr->val, '.');  /* prechecked to exist */
+
+  if (strstr(suffix, ".aiff") || strstr(suffix, ".AIFF"))
+    extlen = 5;
+  else
+    extlen = 4;
+
+  if (strlen(suffix) <= extlen)          /* if true, prechecked == 4 */
+    soundfile = fopen(tptr->val,"rb");
+  else
+    {
+      if ((sscanf(&(suffix[extlen]), "@%hhd%n", &at_n, &ntotal) < 1) || 
+	  (strlen(suffix) != extlen + ntotal) || (at_n < 0))
+	{
+	  printf("Error: Syntax error in sample filename (or in @N specifier).\n");
+	  printtableproblem(tptr, nsl);
+	}
+      suffix[extlen]= '\0';
+      soundfile = fopen(tptr->val,"rb");
+      suffix[extlen]= '@';
+    }
 
   /* AIFF preamble */
 
@@ -2087,36 +2180,54 @@ void aiffileconstcheck(tnode * tptr, tnode * nsl)
     {
       if (!strncmp((char *) cookie,"COMM",4))
 	{
-	  wt_flushsoundbytes(4, &count, tptr, nsl);       /* length */
-	  if (wt_aiffileintval(2, &count, tptr, nsl)!= 1) /* Mono files only */
-	    {
-	      printf("Error: Sample generator can't read stereo AIFF files\n");
-	      printtableproblem(tptr, nsl);
-	    }
+	  chunksize = wt_aiffileintval(4, &count, tptr, nsl);  /* length */
+	  channels = wt_aiffileintval(2, &count, tptr, nsl);  /* 1 = mono, 2 = stereo */
 	  wt_flushsoundbytes(4, &count, tptr, nsl);       /* frames */
-	  sinfo->numbytes = (char)wt_aiffileintval(2, &count, tptr, nsl);
-	  if ((sinfo->numbytes != 8)&&(sinfo->numbytes != 16))
+
+	  sinfo->numbytes = (char)((wt_aiffileintval(2, &count, tptr, nsl)+7)/8);
+	  if ((sinfo->numbytes <= 0) || (sinfo->numbytes > 3)) 
 	    {
-	      printf("Error: Sfront can't handle %i bit AIF file.\n",
-		     sinfo->numbytes);
+	      printf("Error: Sample generator only handles 8/16/24-bit AIF files.\n");
 	      printtableproblem(tptr, nsl);
 	    }
-	  sinfo->numbytes /= 8;
+	  sinfo->framebytes = channels * sinfo->numbytes;
+
+	  if (channels == 1)
+	    sinfo->chanpoint = 0;
+	  else
+	    {
+	      if (at_n >= 0)
+		{
+		  if (at_n > channels - 1)
+		    {
+		      printf("Error: Sample generator @%i filename syntax requests\n"
+			     "       the %i'th channel of an %li channel AIF file.\n", 
+			     at_n, at_n+1, channels);
+		      printtableproblem(tptr, nsl);
+		    }
+		  sinfo->chanpoint = at_n*sinfo->numbytes;
+		}
+	      else
+		sinfo->chanpoint = -1;
+	    }
+
 	  e = ((short)wt_aiffileintval(2, &count, tptr, nsl))&0x7FFF;
 	  m = wt_aiffileintval(4, &count, tptr, nsl);
 	  wt_flushsoundbytes(4, &count, tptr, nsl);
 	  sinfo->srate = (unsigned long) (0.5+(m*exp(log(2)*(e - 16414.0F))));
+	  if (chunksize > 18)  /* '% 2' flushes pad byte */
+	    wt_flushsoundbytes(chunksize - 18 + (chunksize % 2), &count, tptr, nsl);
 	  continue;
       }
 
       if (!strncmp((char *) cookie,"SSND",4))
 	{
-	  /* sinfo->len scaled by numbytes later */
+	  /* sinfo->len scaled by framebytes later */
 	  
 	  sinfo->len = (wt_aiffileintval(4, &count, tptr, nsl) - 8);
 	  wt_flushsoundbytes(8, &count, tptr, nsl);
 	  sinfo->point = count;   /* where sa.c file should fseek() to */
-	  wt_flushsoundbytes(sinfo->len, &count, tptr, nsl);
+	  wt_flushsoundbytes(sinfo->len + (sinfo->len % 2), &count, tptr, nsl);
 	  dataflag = 1;
 	  continue;
 	}
@@ -2124,7 +2235,7 @@ void aiffileconstcheck(tnode * tptr, tnode * nsl)
       if (!strncmp((char *) cookie,"INST",4))
 	{
 	  sinfo->hasbase = 1;
-	  wt_flushsoundbytes(4, &count, tptr, nsl);       /* size */
+	  chunksize = wt_aiffileintval(4, &count, tptr, nsl);  /* size */
 	  unity = wt_aiffileintval(1, &count, tptr, nsl); /* baseNote */
 	  wt_getsoundsbytes(&c,1,&count, tptr, nsl); /* sbytes -> signed */
 	  fract = c;                                      /* detune */
@@ -2140,14 +2251,17 @@ void aiffileconstcheck(tnode * tptr, tnode * nsl)
 	  sinfo->start = wt_aiffileintval(2, &count, tptr, nsl); 
 	  sinfo->end = wt_aiffileintval(2, &count, tptr, nsl);   
 	  wt_flushsoundbytes(6, &count, tptr, nsl);   /* release loop */
+	  if (chunksize > 20)
+	    wt_flushsoundbytes(chunksize - 20 + (chunksize % 2), &count, tptr, nsl);
 	  continue;
 	}
 
       if (!strncmp((char *) cookie,"MARK",4))
 	{
-	  wt_flushsoundbytes(4, &count, tptr, nsl); /* size */
+	  chunksize = wt_aiffileintval(4, &count, tptr, nsl);  /* size */
 	  i = wt_aiffileintval(2, &count, tptr, nsl); /* num markers */
-	  
+	  chunksize = ((chunksize - 2) >= 0) ? (chunksize - 2) : 0;
+
 	  /* inserts marker positions into mhead list */
 	  
 	  while (i--)
@@ -2162,14 +2276,18 @@ void aiffileconstcheck(tnode * tptr, tnode * nsl)
 	      if (!(j&0x0001)) /* pad byte if even */
 		j++;
 	      wt_flushsoundbytes(j, &count, tptr, nsl);
+	      chunksize = ((chunksize - 7 - j) >= 0) ? (chunksize - 7 - j) : 0;
 	    }
+	  if (chunksize)
+	    wt_flushsoundbytes(chunksize + (chunksize % 2), &count, tptr, nsl);
 	  continue;
 	}
-      wt_flushsoundbytes(wt_aiffileintval(4, &count, tptr, nsl),
-			 &count, tptr, nsl);
+
+      chunksize = wt_aiffileintval(4, &count, tptr, nsl);  /* size */
+      wt_flushsoundbytes(chunksize + (chunksize % 2), &count, tptr, nsl);
     }
 
-  sinfo->len = sinfo->len/(sinfo->numbytes);
+  sinfo->len /= sinfo->framebytes;
 
   if (sinfo->hasloop)
     {
@@ -2182,7 +2300,7 @@ void aiffileconstcheck(tnode * tptr, tnode * nsl)
 	      j = 1;
 	      sinfo->start = mptr->position;
 	      if ((unsigned long)(sinfo->start) == sinfo->len)
-		(sinfo->start)--;
+		sinfo->start=0;
 	      break;
 	    }
 	  mptr = mptr->next;
@@ -2198,14 +2316,14 @@ void aiffileconstcheck(tnode * tptr, tnode * nsl)
 	      j = 1;
 	      sinfo->end = mptr->position;
 	      if ((unsigned long)(sinfo->end) == sinfo->len)
-		(sinfo->end)--;
+		sinfo->end=0;
 	      break;
 	    }
 	  mptr = mptr->next;
 	}
       if (!j)
 	wt_bitsounderror(tptr, nsl);      /* can't find end */
-      if (sinfo->start > sinfo->end)      /* behavior in AIFF spec   */
+      if (sinfo->start >= sinfo->end)      /* behavior in AIFF spec   */
 	sinfo->hasloop = 0;
     }
   sinfo->base = (float)(440.0*pow(2.0, ((double)unity +
