@@ -1,26 +1,42 @@
 
-
 /*
 #    Sfront, a SAOL to C translator    
 #    This file: Code generaton: core wavetables
-#    Copyright (C) 1999  Regents of the University of California
 #
-#    This program is free software; you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License (Version 2) as
-#    published by the Free Software Foundation.
+# Copyright (c) 1999-2006, Regents of the University of California
+# All rights reserved.
 #
-#    This program is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#    GNU General Public License for more details.
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are
+# met:
 #
-#    You should have received a copy of the GNU General Public License
-#    along with this program; if not, write to the Free Software
-#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#  Redistributions of source code must retain the above copyright
+#  notice, this list of conditions and the following disclaimer.
+#
+#  Redistributions in binary form must reproduce the above copyright
+#  notice, this list of conditions and the following disclaimer in the
+#  documentation and/or other materials provided with the distribution.
+#
+#  Neither the name of the University of California, Berkeley nor the
+#  names of its contributors may be used to endorse or promote products
+#  derived from this software without specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #
 #    Maintainer: John Lazzaro, lazzaro@cs.berkeley.edu
 */
 
+
 #include "tree.h"
 
 
@@ -236,6 +252,7 @@ void wavegeneratorvar(sigsym * sptr)
   tnode * wname = sptr->defnode->down->next;
   tnode * ident = sptr->defnode->down->next->next->next;
   tnode * tptr =  sptr->defnode->down->next->next->next->next->next->down;
+  sampleinfo * sinfo;
   int i,j;
   int lc = 0;
   char * lname = makewstring(wname->val);
@@ -609,8 +626,12 @@ void wavegeneratorvar(sigsym * sptr)
     if (!(strcmp(ident->val,"sample")))
       {
 	z[lc++]="long %2$s_skip;";
-	z[lc++]="unsigned char %2$s_c[2];";
-	z[lc++]="short %2$s_word;";
+
+	sinfo = (sampleinfo *) 
+	  sptr->defnode->down->next->next->next->next->next->down->next->next->ibus;
+
+	mz(lc);  
+	sprintf(z[lc++],"unsigned char %s_c[%i];",lname, sinfo->framebytes);
 
 	printwavesymblock2(lc, wname); 
 	break;
@@ -721,6 +742,7 @@ void wavegeneratorvar(sigsym * sptr)
 
 extern void createwavetable(sigsym *, char *, int);
 
+
 /*********************************************************/
 /*       high-level routine that creates table code      */
 /*                                                       */
@@ -753,9 +775,14 @@ void createtable(sigsym * ident, char * prefix, int mode)
       internalerror("wtparse.c","createtable");
     if ((ident->tref->assigntot == 0) && (gptr->tref->assigntot == 0))
       {
-	/* define statements alias it to global wavetable */
+	/* hash-define statements alias it to global wavetable */
+
+	if (startupinstr && (currinstrument == startupinstr))
+	  tablestartupcheck(ident, mode, K_IMPORTEXPORT); 
 	break;
       }
+    if (startupinstr && (currinstrument == startupinstr))
+      tablestartupcheck(ident, mode, K_IMPORT); 
     z[lc++]= "i = NT(%1$s).len = gtables[TBL_GBL_%2$s].len;";
     z[lc++]= "NT(%1$s).lenf = gtables[TBL_GBL_%2$s].lenf;";
     z[lc++]= "NT(%1$s).start = gtables[TBL_GBL_%2$s].start;";
@@ -764,11 +791,12 @@ void createtable(sigsym * ident, char * prefix, int mode)
     z[lc++]= "NT(%1$s).stamp = gtables[TBL_GBL_%2$s].stamp;";
     z[lc++]= "NT(%1$s).sr = gtables[TBL_GBL_%2$s].sr;";
     z[lc++]= "NT(%1$s).tend = gtables[TBL_GBL_%2$s].tend;";
-    z[lc++]= "NT(%1$s).m = gtables[TBL_GBL_%2$s].m;";
-    z[lc++]= "NT(%1$s).n = gtables[TBL_GBL_%2$s].n;";
-    z[lc++]= "NT(%1$s).diff = gtables[TBL_GBL_%2$s].diff;";
-    z[lc++]= "NT(%1$s).dmult = gtables[TBL_GBL_%2$s].dmult;";
-    z[lc++]= "NT(%1$s).lmult = gtables[TBL_GBL_%2$s].lmult;";
+    z[lc++]= "NT(%1$s).oconst = gtables[TBL_GBL_%2$s].oconst;";
+    z[lc++]= "NT(%1$s).dint = gtables[TBL_GBL_%2$s].dint;";
+    z[lc++]= "NT(%1$s).dfrac = gtables[TBL_GBL_%2$s].dfrac;";
+    z[lc++]= "NT(%1$s).sffl = gtables[TBL_GBL_%2$s].sffl;";
+    z[lc++]= "NT(%1$s).sfui = gtables[TBL_GBL_%2$s].sfui;";
+    z[lc++]= "NT(%1$s).dsincr = gtables[TBL_GBL_%2$s].dsincr;";
     if ((ident->tref->assigntval == 0) &&
 	(gptr->tref->assigntval == 0))
       {
@@ -784,13 +812,65 @@ void createtable(sigsym * ident, char * prefix, int mode)
     printwaveblock(lc,ident,prefix);
     break;
   case K_IMPORTEXPORT:
-    /* define statements alias it to global wavetable */
+    /* hash-define statements alias it to global wavetable */
+
+    if (startupinstr && (currinstrument == startupinstr))
+      tablestartupcheck(ident, mode, K_IMPORTEXPORT); 
     break;
   default:
     break;
   }
 }
 
+/*********************************************************/
+/*     prints warnings and errors for startup tables     */
+/*     called in createtable() above, and in writeop.c   */
+/*********************************************************/
+
+void tablestartupcheck(sigsym * ident, int mode, int kind)
+
+{
+  switch(kind) {
+  case K_IMPORT:
+    switch (mode) {
+    case S_INSTR:
+      printf("Error: Importing table \"%s\" in startup instrument\n"
+	     "       would cause run-time error, as tables do\n"
+	     "       not exist until after startup's i-cycle.\n\n",
+	     ident->val);
+      showerrorplace(ident->defnode->linenum,
+		     ident->defnode->filename);
+      break;
+    case S_OPCODE:
+      printf("Warning: Importing table \"%s\" in an opcode called\n" 
+	     "         by the startup instrument may cause a run-time\n"
+	     "         error, as tables do not exist during startup's i-cycle.\n\n",
+	     ident->val);
+      break;
+    }
+    break;
+  case K_IMPORTEXPORT:
+    switch (mode) {
+    case S_INSTR:
+      printf("Warning: %s table \"%s\" in startup instrument\n"
+	     "         may cause run-time error, as tables do\n"
+	     "         not exist until after startup's i-cycle.\n\n",
+	     ident->kind == K_IMPORT ? "Importing" : "Import/export of",
+	     ident->val);
+      break;
+    case S_OPCODE:
+      printf("Warning: %s table \"%s\" in an opcode called\n" 
+	     "         by the startup instrument may cause a run-time\n"
+	     "         error, as tables do not exist during startup's i-cycle.\n\n",
+	     ident->kind == K_IMPORT ? "Importing" : "Import/export of",
+	     ident->val);
+      break;
+    }
+    break;
+  }
+  return;
+}
+
 
 /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
 /*                                                              */
@@ -2231,12 +2311,13 @@ void tablepreamble(sigsym * ident, char * prefix, int mode, int action)
       z[lc++]= "NT(%1$s).start = 0;";
       z[lc++]= "NT(%1$s).end = 0;";
       z[lc++]= "NT(%1$s).sr = 0;";
-      z[lc++]= "NT(%1$s).dmult = 0;";
-      z[lc++]= "NT(%1$s).lmult = 0;";
+      z[lc++]= "NT(%1$s).dint = 0;";
+      z[lc++]= "NT(%1$s).dfrac = 0;";
+      z[lc++]= "NT(%1$s).sffl = 0;";
+      z[lc++]= "NT(%1$s).sfui = 0;";
+      z[lc++]= "NT(%1$s).dsincr = 0;";
       z[lc++]= "NT(%1$s).base = 0;";
-      z[lc++]= "NT(%1$s).m = 0;";
-      z[lc++]= "NT(%1$s).n = 0;";
-      z[lc++]= "NT(%1$s).diff = 0;";
+      z[lc++]= "NT(%1$s).oconst = 0;";
       z[lc++]= "NT(%1$s).tend = 0;";
     }
 
@@ -2333,12 +2414,18 @@ void tablepreamble(sigsym * ident, char * prefix, int mode, int action)
     internalerror("wtparse.c", "case statement");
   }
 
+  if (interp == INTERP_SINC)
+    {
+      z[lc++]= "NT(%1$s).sffl = 1.0F;";
+      z[lc++]= "NT(%1$s).sfui = 0x00010000;";
+      z[lc++]= "NT(%1$s).dsincr = SINC_PILEN;";
+    }
+
   z[lc++]= "NT(%1$s).stamp = scorebeats;";
   z[lc++]= "NT(%1$s).lenf = (float)(i);";
+  z[lc++]= "NT(%1$s).oconst = ATIME*((float)i);";
   z[lc++]= "if (i>0)";
   z[lc++]= "{";
-  z[lc++]= "  NT(%1$s).n = (i - 1.0F)/i;";
-  z[lc++]= "  NT(%1$s).diff = (i - 1.0F)/i;";
   z[lc++]= "  NT(%1$s).tend = i - 1;";
   z[lc++]= "}";
   z[lc++]= "NT(%1$s).t = (float *) malloc((i+1)*sizeof(float));";
@@ -2442,6 +2529,7 @@ void samplefilecode(sigsym * ident, char * prefix, int * lcptr)
   char newval[STRSIZE];
   int lc = *lcptr;
   sampleinfo * sinfo;
+  double intdummy;
   tnode * sizeptr;
   tnode * fileptr;
   tnode * skipptr = NULL;
@@ -2450,6 +2538,8 @@ void samplefilecode(sigsym * ident, char * prefix, int * lcptr)
   unsigned long skip = 0;
   int known, currblocksafe;
   char * lname = makewstring(ident->val);
+  char * suffix;
+  char bytecount;
 
   /* see wtconst.c for regularization of constant size and skip */
 
@@ -2468,6 +2558,51 @@ void samplefilecode(sigsym * ident, char * prefix, int * lcptr)
   mz(lc); sprintf(z[lc++], "NT(%s_%s).sr = %li;", prefix, ident->val,
 		  sinfo->srate);
 
+  if (srate == sinfo->srate)
+    {
+      mz(lc); sprintf(z[lc++], "NT(%s_%s).dint = 1;", prefix, ident->val);
+      mz(lc); sprintf(z[lc++], "NT(%s_%s).dfrac = 0;", prefix, ident->val);
+    }
+  else
+    {
+      mz(lc); sprintf(z[lc++], "NT(%s_%s).dfrac = %luUL;", prefix, ident->val,
+		      (unsigned long) (4294967296.0*
+				       modf(((double)sinfo->srate)/((double)srate), 
+					    &intdummy)));
+      mz(lc); sprintf(z[lc++], "NT(%s_%s).dint = %luUL;", prefix, ident->val,
+		      (unsigned long) intdummy);
+    }
+
+  if (interp == INTERP_SINC)
+    {
+      if (srate >= sinfo->srate)
+	{
+	  mz(lc); sprintf(z[lc++], "NT(%s_%s).sffl = 1.0F;", prefix, ident->val);
+	  mz(lc); sprintf(z[lc++], "NT(%s_%s).sfui = 0x00010000;", prefix, ident->val);
+	  mz(lc); sprintf(z[lc++], "NT(%s_%s).dsincr = SINC_PILEN;", prefix, ident->val);
+	}
+      else
+	{
+	  if (srate*sinc_upmax > sinfo->srate)
+	    {
+	      mz(lc); sprintf(z[lc++], "NT(%s_%s).sffl = (ARATE/NT(%s_%s).sr);", 
+			      prefix, ident->val, prefix, ident->val);
+	    }
+	  else
+	    {
+	      mz(lc); sprintf(z[lc++], "NT(%s_%s).sffl = (1.0F/SINC_UPMAX);", 
+			      prefix, ident->val);
+	    }
+
+	  mz(lc); sprintf(z[lc++], 
+			  "NT(%s_%s).sfui = ((float)(pow(2,16)))*NT(%s_%s).sffl + 0.5F;", 
+			  prefix, ident->val, prefix, ident->val);
+	  mz(lc); sprintf(z[lc++], 
+			  "NT(%s_%s).dsincr = (SINC_PILEN*NT(%s_%s).sfui) >> 16;", 
+			  prefix, ident->val, prefix, ident->val);
+	}
+    }
+
   if (sinfo->hasbase)
     {      
       mz(lc); sprintf(z[lc++], "NT(%s_%s).base = %s;", prefix, ident->val,
@@ -2480,8 +2615,8 @@ void samplefilecode(sigsym * ident, char * prefix, int * lcptr)
 		      prefix, ident->val, sinfo->start);
       if (sinfo->end)
 	{
-	  mz(lc); sprintf(z[lc++], "NT(%s_%s).tend = %li;", prefix,
-			  ident->val, sinfo->end);
+	  mz(lc); sprintf(z[lc++], "NT(%s_%s).tend = NT(%s_%s).end = %li;", 
+			  prefix, ident->val, prefix, ident->val, sinfo->end);
 	}
     }
 
@@ -2511,57 +2646,24 @@ void samplefilecode(sigsym * ident, char * prefix, int * lcptr)
 		      "NT(%s_%s).lenf = (float)(NT(%s_%s).len = %li);",
 		      prefix,ident->val, prefix, ident->val, size);
 
+      mz(lc); sprintf(z[lc++], "NT(%s_%s).oconst = NT(%s_%s).lenf*ATIME;",
+		      prefix,ident->val, prefix, ident->val);
+
       mz(lc); sprintf(z[lc++], "NT(%s_%s).t = (float *)(calloc(%li, %i));",
 		      prefix, ident->val, size + 1, (int)sizeof(float));
 
-      mz(lc); sprintf(z[lc++], "NT(%s_%s).dmult = %eF;", prefix, ident->val,
-		      (double)(((1.0F/srate)*(sinfo->srate))/size));
-
-      if (sinfo->hasbase)
-	{      
-	  mz(lc); sprintf(z[lc++], "NT(%s_%s).lmult = %eF;",
-			  prefix,ident->val,  (double)
-			  (((1.0F/srate)*sinfo->srate)/(size*sinfo->base)));
-	}
-      else
-	{
-	  mz(lc); sprintf(z[lc++], "NT(%s_%s).lmult = %eF;", 
-			  prefix, ident->val, (double) 
-			  (((float)srate)/(sinfo->srate)));
-	}
-
       if (sinfo->hasloop)
 	{	  
-	  mz(lc); sprintf(z[lc++], "NT(%s_%s).m = %eF;", prefix, ident->val,
-			  (double)(((float)(sinfo->start))/size));
-	  if (sinfo->end)
-	    {
-	      /* tend independent of size and skip, assigned earlier */
-
-	      mz(lc); sprintf(z[lc++], "NT(%s_%s).n = %eF;", prefix,
-			      ident->val, ((double)(sinfo->end))/size);
-	      mz(lc); sprintf(z[lc++], "NT(%s_%s).diff = %eF;",
-			      prefix,ident->val,
-			      ((double)(sinfo->end) - sinfo->start)/size);
-	    }
-	  else
+	  if (!(sinfo->end))
 	    {
 	      mz(lc); sprintf(z[lc++], "NT(%s_%s).tend = %li;",
 			      prefix,ident->val, size - 1);
-	      mz(lc); sprintf(z[lc++], "NT(%s_%s).n = %eF;",
-			      prefix,ident->val, (double)((size - 1.0)/size));
-	      mz(lc); sprintf(z[lc++], "NT(%s_%s).diff = %eF;",
-			      prefix,ident->val, (double)
-			      ((size - 1.0F - sinfo->start)/size));
 	    }
 	}
       else
 	{
 	  mz(lc); sprintf(z[lc++], "NT(%s_%s).tend = %li;",
 			  prefix,ident->val, size - 1);
-	  mz(lc); sprintf(z[lc++], "NT(%s_%s).n = NT(%s_%s).diff = %eF;",
-			  prefix,ident->val, prefix,ident->val, 
-			  (double)((size - 1.0F)/size));
 	}
     }
   else
@@ -2598,43 +2700,22 @@ void samplefilecode(sigsym * ident, char * prefix, int * lcptr)
 		      prefix, ident->val, prefix, ident->val, lname,
 		      sinfo->len, lname, lname);
 
+      mz(lc); sprintf(z[lc++], "NT(%s_%s).oconst = NT(%s_%s).lenf*ATIME;",
+		      prefix,ident->val, prefix, ident->val);
+
       z[lc++]= "NT(%1$s).t = (float *)(calloc(NT(%1$s).len +1,";
       z[lc++]= "                       sizeof(float)));";
 
-      z[lc++]= "NT(%1$s).dmult = NT(%1$s).sr*ATIME/NT(%1$s).lenf;";
-
-      if (sinfo->hasbase)
-	{      
-	  mz(lc); sprintf(z[lc++], "NT(%s_%s).lmult = NT(%s_%s).dmult*%eF;",
-			  prefix,ident->val, prefix,ident->val,
-			  (double)(1.0F/(sinfo->base)));
-	}
-      else
-	{
-	  z[lc++]= "NT(%1$s).lmult = (1.0F/NT(%1$s).len)*ATIME;";
-	}
       if (sinfo->hasloop)
 	{
-	  mz(lc); sprintf(z[lc++], "NT(%s_%s).m = %eF/NT(%s_%s).lenf;",
-			  prefix, ident->val, (double)(sinfo->start),
-			  prefix, ident->val);
-	  if (sinfo->end)
-	    {
-	      mz(lc); sprintf(z[lc++], "NT(%s_%s).n = %eF/NT(%s_%s).lenf;",
-			      prefix, ident->val, (double)(sinfo->end),
-			      prefix, ident->val);
-	    }
-	  else
+	  if (!(sinfo->end))
 	    {
 	      z[lc++]="NT(%1$s).tend = NT(%1$s).len - 1;";
-	      z[lc++]="NT(%1$s).n = 1.0F-(1.0F/NT(%1$s).len);";
 	    }
-	  z[lc++]= "    NT(%1$s).diff = NT(%1$s).n - NT(%1$s).m;";
 	}
       else
 	{
 	  z[lc++]="NT(%1$s).tend = NT(%1$s).len - 1;";
-	  z[lc++]="NT(%1$s).n = NT(%1$s).diff = 1.0F-(1.0F/NT(%1$s).len);";
 	}
     }
 
@@ -2642,7 +2723,14 @@ void samplefilecode(sigsym * ident, char * prefix, int * lcptr)
   /* code for reading data from file */
   /***********************************/
 
+  if ((suffix = strrchr(fileptr->val, '.')) && (suffix = strrchr(suffix, '@')))
+    suffix[0] = '\0';
+
   mz(lc); sprintf(z[lc++], "sfile = fopen(\"%s\", \"rb\");", fileptr->val);
+
+  if (suffix)
+    suffix[0] = '@';
+
   z[lc++]= "if (sfile == NULL)";
   genex(&lc, ident->defnode->down, "Samplefile not found");
 
@@ -2651,7 +2739,7 @@ void samplefilecode(sigsym * ident, char * prefix, int * lcptr)
       /* we don't fseek() to simplify signal handling for real-time */
 
       mz(lc); sprintf(z[lc++], "for (i = 0; i < %li; i++)", 
-		      sinfo->point + skip*sinfo->numbytes);
+		      sinfo->point + skip*sinfo->framebytes);
       z[lc++]= "if (!rread(%2$s_c,1,1,sfile))";
       genex(&lc, ident->defnode->down, "Corrupt samplefile");
 
@@ -2665,7 +2753,7 @@ void samplefilecode(sigsym * ident, char * prefix, int * lcptr)
     {
 
       mz(lc); sprintf(z[lc++], "for (i = 0; i < %li + %i*%s_skip; i++)", 
-		      sinfo->point, sinfo->numbytes, lname);
+		      sinfo->point, sinfo->framebytes, lname);
       z[lc++]= "if (!rread(%2$s_c,1,1,sfile))";
       genex(&lc, ident->defnode->down, "Corrupt samplefile");
 
@@ -2677,30 +2765,166 @@ void samplefilecode(sigsym * ident, char * prefix, int * lcptr)
     }
 
   mz(lc); sprintf(z[lc++], "if (!rread(%s_c,1,%i,sfile))", 
-		  lname, sinfo->numbytes);
+		  lname, sinfo->framebytes);
   genex(&lc, ident->defnode->down, "Corrupt samplefile");
 
   if (sinfo->wav)
     {
-      if (sinfo->numbytes == 2)
-	{	
-	  z[lc++]= "     %2$s_word = (short)%2$s_c[0];";
-	  z[lc++]= "     %2$s_word |= (short)%2$s_c[1]<<8;";
-	  z[lc++]= "     NT(%1$s).t[i] = 3.051758e-5F*%2$s_word;";
+      if (sinfo->chanpoint > -1)
+	{
+	  switch (sinfo->numbytes) {
+	  case 3:
+	    z[lc++]= "     NT(%1$s).t[i] = ((float)pow(2, -23))*(";
+	    mz(lc);  sprintf(z[lc++], "    ((long)(%s_c[%hhi])) + ", 
+			     lname, sinfo->chanpoint);
+	    mz(lc);  sprintf(z[lc++], "    (((long)(%s_c[%hhi])) << 8) +", 
+			     lname, sinfo->chanpoint + 1);
+	    mz(lc);  sprintf(z[lc++], "    (((long)((char)(%s_c[%hhi]))) << 16));", 
+			     lname, sinfo->chanpoint + 2);
+	    break;
+	  case 2:
+	    z[lc++]= "     NT(%1$s).t[i] = ((float)pow(2, -15))*(";
+	    mz(lc);  sprintf(z[lc++], "    ((long)(%s_c[%hhi])) + ", 
+			     lname, sinfo->chanpoint);
+	    mz(lc);  sprintf(z[lc++], "    (((long)((char)(%s_c[%hhi]))) << 8));", 
+			     lname, sinfo->chanpoint + 1);
+	    break;
+	  case 1:
+	    z[lc++]= "     NT(%1$s).t[i] = ((float)pow(2, -7))*(";
+	    mz(lc);  sprintf(z[lc++], "    ((short)(%s_c[%hhi])) - 128); ", 
+			     lname, sinfo->chanpoint);
+	    break;
+	  }
 	}
       else
-	z[lc++]= "   NT(%1$s).t[i] = 7.8125e-3F*%2$s_c[0] - 1.0F;";
+	{
+	  switch (sinfo->numbytes) {
+	  case 3:
+	    z[lc++]= "     NT(%1$s).t[i] = ((float)pow(2, -23))*(";
+	    
+	    mz(lc);  sprintf(z[lc++], "     1.0F/%i.0F)*(", 
+			     (int)(sinfo->framebytes/sinfo->numbytes));
+	    break;
+	  case 2:
+	    z[lc++]= "     NT(%1$s).t[i] = ((float)pow(2, -15))*(";
+	    
+	    mz(lc);  sprintf(z[lc++], "     1.0F/%i.0F)*(", 
+			     (int)(sinfo->framebytes/sinfo->numbytes));
+	    break;
+	  case 1:
+	    z[lc++]= "     NT(%1$s).t[i] = ((float)pow(2, -7))*(";
+	    
+	    mz(lc);  sprintf(z[lc++], "     1.0F/%i.0F)*(", 
+			     (int)(sinfo->framebytes/sinfo->numbytes));
+	    break;
+	  }
+
+	  bytecount = 0;
+	  while (bytecount < sinfo->framebytes)
+	    {
+	      switch (sinfo->numbytes) {
+	      case 3:
+		mz(lc);  sprintf(z[lc++], "    %c ((long)(%s_c[%hhi])) + ", 
+				 bytecount ? '+' : ' ', lname, bytecount);
+		mz(lc);  sprintf(z[lc++], "    (((long)(%s_c[%hhi])) << 8) +", 
+				 lname, bytecount + 1);
+		mz(lc);  sprintf(z[lc++], "    (((long)((char)(%s_c[%hhi]))) << 16)", 
+				 lname, bytecount + 2);
+		break;
+	      case 2:
+		mz(lc);  sprintf(z[lc++], "    %c ((long)(%s_c[%hhi])) + ", 
+				 bytecount ? '+' : ' ', lname, bytecount);
+		mz(lc);  sprintf(z[lc++], "    (((long)((char)(%s_c[%hhi]))) << 8)", 
+				 lname, bytecount + 1);
+		break;
+	      case 1:
+		mz(lc);  sprintf(z[lc++], "    %c ((short)(%s_c[%hhi])) - 128 ", 
+				 bytecount ? '+' : ' ', lname, bytecount);
+		break;
+	      }
+	      bytecount += sinfo->numbytes;
+	    }
+	  z[lc++]= "     );";
+	}
     }
   else
     {
-      if (sinfo->numbytes == 2)
+      if (sinfo->chanpoint > -1)
 	{
-	  z[lc++]= "     %2$s_word = (short)%2$s_c[1];";
-	  z[lc++]= "     %2$s_word |= (short)%2$s_c[0]<<8;";
-	  z[lc++]= "     NT(%1$s).t[i] = 3.051758e-5F*%2$s_word;";
+	  switch (sinfo->numbytes) {
+	  case 3:
+	    z[lc++]= "     NT(%1$s).t[i] = ((float)pow(2, -23))*(";
+	    mz(lc);  sprintf(z[lc++], "    ((long)(%s_c[%hhi])) + ", 
+			     lname, sinfo->chanpoint + 2);
+	    mz(lc);  sprintf(z[lc++], "    (((long)(%s_c[%hhi])) << 8) +", 
+			     lname, sinfo->chanpoint + 1);
+	    mz(lc);  sprintf(z[lc++], "    (((long)((char)(%s_c[%hhi]))) << 16));", 
+			     lname, sinfo->chanpoint);
+	    break;
+	  case 2:
+	    z[lc++]= "     NT(%1$s).t[i] = ((float)pow(2, -15))*(";
+	    mz(lc);  sprintf(z[lc++], "    ((long)(%s_c[%hhi])) + ", 
+			     lname, sinfo->chanpoint + 1);
+	    mz(lc);  sprintf(z[lc++], "    (((long)((char)(%s_c[%hhi]))) << 8));", 
+			     lname, sinfo->chanpoint);
+	    break;
+	  case 1:
+	    z[lc++]= "     NT(%1$s).t[i] = ((float)pow(2, -7))*(";
+	    mz(lc);  sprintf(z[lc++], "    ((short)(%s_c[%hhi]))); ", 
+			     lname, sinfo->chanpoint);
+	    break;
+	  }
 	}
       else
-	z[lc++]= "   NT(%1$s).t[i] = 7.8125e-3F*(signed char)(%2$s_c[0]);";
+	{
+	  switch (sinfo->numbytes) {
+	  case 3:
+	    z[lc++]= "     NT(%1$s).t[i] = ((float)pow(2, -23))*(";
+
+	    mz(lc);  sprintf(z[lc++], "     1.0F/%i.0F)*(", 
+			     (int)(sinfo->framebytes/sinfo->numbytes));
+	    break;
+	  case 2:
+	    z[lc++]= "     NT(%1$s).t[i] = ((float)pow(2, -15))*(";
+	    
+	    mz(lc);  sprintf(z[lc++], "     1.0F/%i.0F)*(", 
+			     (int)(sinfo->framebytes/sinfo->numbytes));
+	    break;
+	  case 1:
+	    z[lc++]= "     NT(%1$s).t[i] = ((float)pow(2, -7))*(";
+	    
+	    mz(lc);  sprintf(z[lc++], "     1.0F/%i.0F)*(", 
+			     (int)(sinfo->framebytes/sinfo->numbytes));
+	    break;
+	  }
+
+	  bytecount = 0;
+	  while (bytecount < sinfo->framebytes)
+	    {
+	      switch (sinfo->numbytes) {
+	      case 3:
+		mz(lc);  sprintf(z[lc++], "    %c ((long)(%s_c[%hhi])) + ", 
+				 bytecount ? '+': ' ', lname, bytecount + 2);
+		mz(lc);  sprintf(z[lc++], "    (((long)(%s_c[%hhi])) << 8) +", 
+				 lname, bytecount + 1);
+		mz(lc);  sprintf(z[lc++], "    (((long)((char)(%s_c[%hhi]))) << 16)", 
+				 lname, bytecount);
+		break;
+	      case 2:
+		mz(lc);  sprintf(z[lc++], "    %c ((long)(%s_c[%hhi])) + ", 
+				 bytecount ? '+': ' ', lname, bytecount + 1);
+		mz(lc);  sprintf(z[lc++], "    (((long)((char)(%s_c[%hhi]))) << 8)", 
+				 lname, bytecount);
+		break;
+	      case 1:
+		mz(lc);  sprintf(z[lc++], "    %c ((short)(%s_c[%hhi])) ", 
+				 bytecount ? '+': ' ', lname, bytecount);
+		break;
+	      }
+	      bytecount += sinfo->numbytes;
+	    }
+	  z[lc++]= "     );";
+	}
     }	
 
   z[lc++]= "}";
